SystemOrganization addCategory: #'Dominion-Cards'!
SystemOrganization addCategory: #Dominion!
SystemOrganization addCategory: #'Dominion-Display'!
SystemOrganization addCategory: #'Dominion-Test'!

OrderedCollection subclass: #CardArea
	instanceVariableNames: 'visibility'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!CardArea class methodsFor: 'instance creation' stamp: 'JohnParker 7/17/2012 21:15'!
visibility: aSymbol
	^self new visibility: aSymbol.! !

!CardArea methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 21:16'!
visibility 
	^visibility.! !

!CardArea methodsFor: 'initialize-release' stamp: 'JohnParker 7/17/2012 21:45'!
visibility: symbol1
	visibility := symbol1.! !

StandardWindow subclass: #GameTable
	instanceVariableNames: 'supply kingdom playerArea turnLogs played scrollPane'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!GameTable commentStamp: 'MichaelOSullivan 7/15/2012 14:08' prior: 0!
GameTable represents an area where players gather to play a game of Dominion. All cards are kept and played on the game table.!

!GameTable methodsFor: 'initialize' stamp: 'MichaelOSullivan 7/18/2012 22:15'!
initialize
	super initialize.
	self setLabel: 'Dominion'.! !

!GameTable methodsFor: 'initialize' stamp: 'MichaelOSullivan 7/22/2012 16:30'!
initializeWith: aBoard
	| width rect |
	scrollPane := ScrollPane new.
	self addMorph: scrollPane frame: (0@0 corner: 1@1).
	supply := Supply new initializeWith: aBoard.
	kingdom := Kingdom new initializeWith: aBoard.
	playerArea := PlayerArea new setDeck: (aBoard deck: 1) board: aBoard. "TODO: deck number needs to be dynamic"
	played := InPlayArea new setDeck: (aBoard deck: 1). "TODO: deck number needs to be dynamic"
	
	width := supply width max: kingdom width.
	
	supply position: scrollPane layoutBounds topLeft.
	supply width: width.	
	scrollPane scroller addMorph: supply.
	
	kingdom topLeft: supply bottomLeft.
	kingdom width: width.
	scrollPane scroller addMorph: kingdom.
	
	playerArea bottomLeft: kingdom bottomRight.
	scrollPane scroller addMorph: playerArea.

	played extent: playerArea extent.
	played bottomLeft: playerArea topLeft.
	scrollPane scroller addMorph: played.
	
	turnLogs := TurnLogArea new.
	turnLogs topLeft: supply topRight.
	rect := Rectangle
				origin: turnLogs bounds origin
				corner: played topRight.
	turnLogs bounds: rect.
	turnLogs initializeWith: aBoard.
	scrollPane scroller addMorph: turnLogs.! !

BorderedMorph subclass: #InPlayArea
	instanceVariableNames: 'deck cards'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!InPlayArea methodsFor: 'geometry' stamp: 'MichaelOSullivan 7/20/2012 15:21'!
extent: aPoint
	super extent: aPoint.
	cards extent: (aPoint translateBy: -20 @ -20).! !

!InPlayArea methodsFor: 'initailize' stamp: 'MichaelOSullivan 7/20/2012 15:21'!
initialize
	super initialize.
	self color: (Color r: 0.8 g: 0.6 b: 0.3).
	cards := BorderedMorph new.
	cards
		borderInset;
		color: self color;
		position: (self innerBounds origin translateBy: 10@10).
	self addMorph: cards.! !

!InPlayArea methodsFor: 'initailize' stamp: 'MichaelOSullivan 7/20/2012 14:57'!
setDeck: aDeck
	deck := aDeck.! !

!InPlayArea methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/20/2012 14:58'!
update
	| pos rect sortedCards card |
	
	self removeAllMorphs.
	pos := self innerBounds origin translateBy: 10@0.
	sortedCards := deck self asSortedCollection: [:a :b | a cardName < b cardName].
	sortedCards do: [:each |	card := CardMorph new setCardType: each class count: ''.
								card position: pos.
								self addMorph: card.
								pos := pos translateBy: (card width + 10)@10 ].
	rect := Rectangle 	origin: (self innerBounds origin)
				    		corner: (card bottomRight translateBy: 10@10).
	self bounds: rect.! !

BorderedMorph subclass: #Kingdom
	instanceVariableNames: 'title cards board actionSet'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!Kingdom methodsFor: 'card management' stamp: 'MichaelOSullivan 7/18/2012 22:47'!
addCard: aCardClass
	| card row |

	(aCardClass inheritsFrom: ActionCard) ifTrue: [actionSet add: aCardClass].! !

!Kingdom methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/17/2012 21:29'!
board
	^board! !

!Kingdom methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/17/2012 21:28'!
cards
	^cards! !

!Kingdom methodsFor: 'initialize' stamp: 'MichaelOSullivan 7/20/2012 13:34'!
initializeWith: aBoard
	| cardClasses row card |
	super initialize.
	board := aBoard.
	cards := Dictionary new.
	actionSet := Set new.
	
	board cards do: [:each | self addCard: each].
	
	actionSet do: [:each | card := CardMorph new setCardType: each count: (aBoard cardsRemaining: each).
		self addMorph: card.
		(cards includesKey: each cost)
			ifTrue: [row := cards at: each cost.
					row add: card.]
			ifFalse: [row := OrderedCollection new.
					row add: card.
					cards at: each cost
					 	    put: row]		].
			
	
	self color: (Color r: 0.8 g: 0.6 b: 0.3).
	title := StringMorph new contents: 'Kingdom'.
	self addMorph: title.
	self render! !

!Kingdom methodsFor: 'drawing' stamp: 'MichaelOSullivan 7/18/2012 19:52'!
render
	(KingdomRenderer new initializeWith: self) compute! !

!Kingdom methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/17/2012 21:28'!
title
	^title! !

BorderedMorph subclass: #PlayerArea
	instanceVariableNames: 'board deck hand actionLbl actions buyLbl buys fundsLbl funds drawLbl drawPile discardLbl discardPile buyButton cleanupButton'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!PlayerArea methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/22/2012 22:06'!
cleanup
	"Expected to be triggered when the Cleanup button is clicked"
	
	board execute: (EndTurnCommand new deck: deck number).
	self update.
	! !

!PlayerArea methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/22/2012 16:33'!
cleanup: args
	"Expected to be triggered when the Cleanup button is clicked"
	
	board execute: (EndTurnCommand new deck: deck number).
	! !

!PlayerArea methodsFor: 'initialize' stamp: 'MichaelOSullivan 7/22/2012 16:21'!
initialize
	super initialize.
	self color: (Color r: 0.8 g: 0.6 b: 0.3).
	actionLbl := StringMorph new contents: 'Actions:'.
	self addMorph: actionLbl.
	actions := StringMorph new contents: '-1'.
	self addMorph: actions.
	buyLbl := StringMorph new contents: 'Buys:'.
	self addMorph: buyLbl.
	buys := StringMorph new contents: '-1'.
	self addMorph: buys.
	fundsLbl := StringMorph new contents: 'Funds:'.
	self addMorph: fundsLbl.
	funds := StringMorph new contents: '-1'.
	self addMorph: funds.
	drawLbl := StringMorph new contents: 'Draw Pile:'.
	self addMorph: drawLbl.
	drawPile := StringMorph new contents: '-1'.
	self addMorph: drawPile.
	discardLbl := StringMorph new contents: 'Discard Pile:'.
	self addMorph: discardLbl.
	discardPile := StringMorph new contents: '-1'.
	self addMorph: discardPile.
	
	buyButton := SimpleButtonMorph new label: 'Buy'.
	buyButton 	color: Color lightGray;
				borderWidth: 3;
				doesBevels;
				borderColor: #raised.
	self addMorph: buyButton.
	
	cleanupButton := SimpleButtonMorph new label: 'Cleanup'.
	cleanupButton color: Color lightGray;
					borderWidth: 3;
					doesBevels;
					borderColor: #raised;
					target: self;
					actionSelector: #cleanup.
	self addMorph: cleanupButton.
	
	hand := BorderedMorph new.
	hand
		borderInset;
		color: self color.
	self addMorph: hand.
	
	self render.! !

!PlayerArea methodsFor: 'drawing' stamp: 'MichaelOSullivan 7/20/2012 15:05'!
render
	actionLbl position: (self innerBounds origin translateBy: 10@10).
	actions topLeft: (actionLbl topRight translateBy: 10@0).
	buyLbl topLeft: (actionLbl bottomLeft translateBy: 0@10).
	buys topLeft: (actions bottomLeft translateBy: 0@10).
	fundsLbl topLeft: (buyLbl bottomLeft translateBy: 0@10).
	funds topLeft: (buys bottomLeft translateBy: 0@10).
	discardLbl topLeft: (actions topRight translateBy: 20@0).
	discardPile topLeft: (discardLbl topRight translateBy: 10@0).
	drawLbl topLeft: (buys topRight translateBy: 20@0).
	drawPile topLeft: (discardPile bottomLeft translateBy: 0@10).
	buyButton topLeft: (fundsLbl bottomLeft translateBy: 0@20).
	cleanupButton topLeft: (buyButton topRight translateBy: 10@0).
	
	hand topLeft: (discardPile topRight translateBy: 10@0).
	
	self update! !

!PlayerArea methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/22/2012 16:30'!
setDeck: aDeck board: aBoard
	deck := aDeck.
	board := aBoard.
	self update.! !

!PlayerArea methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/20/2012 15:06'!
update
	| card pos sortedCards rect |
	
	deck isNil ifTrue: [^self].
	
	actions contents: deck actions asString.
	buys contents: deck buys asString.
	funds contents: '$', deck money asString.
	drawPile contents: deck drawCount asString.
	discardPile contents: deck discardCount asString.
	
	pos := hand innerBounds origin translateBy: 10@10.
	hand removeAllMorphs.
	sortedCards := deck hand asSortedCollection: [:a :b | a cardName < b cardName].
	sortedCards do: [:each |	card := CardMorph new setCardType: each class count: ''.
								card position: pos.
								hand addMorph: card.
								pos := pos translateBy: (card width + 10)@0 ].
	rect := Rectangle 	origin: (hand innerBounds origin)
				    		corner: (card bottomRight translateBy: 10@10).
	hand bounds: rect.
	
	rect := Rectangle 	origin: (self innerBounds origin)
				    		corner: ((hand topRight x @ hand bottomLeft y) translateBy: 10@10).
	self bounds: rect.! !

BorderedMorph subclass: #Supply
	instanceVariableNames: 'title board victory treasure victorySet treasureSet'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!Supply commentStamp: 'MichaelOSullivan 7/15/2012 14:04' prior: 0!
The Supply is the area where the Victory and Treasure cards are placed.!

!Supply methodsFor: 'card management' stamp: 'MichaelOSullivan 7/16/2012 18:50'!
addCard: aCardClass
	(aCardClass superclass = VictoryCard) ifTrue: [victorySet add: aCardClass].
	(aCardClass superclass = TreasureCard) ifTrue: [treasureSet add: aCardClass].
	! !

!Supply methodsFor: 'initialize' stamp: 'MichaelOSullivan 7/18/2012 20:58'!
initializeWith: aBoard
	super initialize.
	board := aBoard.
	victory := OrderedCollection new.
	treasure := OrderedCollection new.
	victorySet := Set new.
	treasureSet := Set new.
	title := StringMorph new.
	title contents: 'The Supply'.
	self addMorph: title.
	board cards do: [:each | self addCard: each].
	self render! !

!Supply methodsFor: 'drawing' stamp: 'MichaelOSullivan 7/20/2012 13:36'!
render
	| position x y cardHeight card rect sortedTreasure sortedVictory|
	
	self color: (Color r: 0.8 g: 0.6 b: 0.3).
	cardHeight := 0.
	title position: (self innerBounds origin) + (5 @ 5).
	position := (title position) + (0 @ (title height + 20)).
	x := position x.
	y := position y.

	sortedTreasure := treasureSet asSortedCollection: [:a :b | a cost < b cost].
	sortedTreasure do: [:each |	card := CardMorph new setCardType: each count: (board cardsRemaining: each).
									treasure add: card.
									self addMorph: card. ].
	
	treasure do: [:each |	each position: (x @ y).
							cardHeight := each height.
							x := x + each width + 20 ].
	x := position x.
	y := y + cardHeight + 20.
	
	sortedVictory := victorySet asSortedCollection: [:a :b | a cost < b cost].
	sortedVictory do: [:each | 	card := CardMorph new setCardType: each count: (board cardsRemaining: each).
								victory add: card.
								self addMorph: card. ].
	
	victory do: [:each |	each position: (x @ y).
						cardHeight := each height.
						x := x + each width + 20 ].
				
	y := y + cardHeight + 20.
	rect := Rectangle 	origin: (self innerBounds origin)
				    		corner: (x @ y).
	self bounds: rect! !

BorderedMorph subclass: #TurnLogArea
	instanceVariableNames: 'logs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!TurnLogArea methodsFor: 'drawing' stamp: 'MichaelOSullivan 7/21/2012 11:03'!
drawOn: aCanvas
	self render.
	super drawOn: aCanvas.! !

!TurnLogArea methodsFor: 'initialize' stamp: 'MichaelOSullivan 7/22/2012 15:26'!
initialize: aScrollPane
	aScrollPane
		color: Color white;
		borderInset;
		borderWidth: 10.
	self addMorph: aScrollPane.
	! !

!TurnLogArea methodsFor: 'initialize' stamp: 'MichaelOSullivan 7/22/2012 16:08'!
initializeWith: aBoard
	logs := Array new: aBoard numberOfPlayers.
	1 to: logs size do: [:each | logs at: each put: TurnLog new].
	logs withIndexDo: [:each :index | 	aBoard addObserver: each at: index.
										each setDeck: index.
										self initialize: each].
	self color: (Color r: 0.8 g: 0.6 b: 0.3).
	self render.! !

!TurnLogArea methodsFor: 'drawing' stamp: 'MichaelOSullivan 7/21/2012 11:04'!
render
	| aRect x y locations |

	x := self bounds extent x / 3.
	y := self bounds extent y / 3.
	aRect := self innerBounds origin extent:(x @ y translateBy: -10 @ -10).
	
	self setLogSizes: aRect.
	
	locations := Array new: 4.
	locations at: 1 put: x @ (2 * y).
	locations at: 2 put: 0 @ y.
	locations at: 3 put: x @ 0.
	locations at: 4 put: (2 * x) @ y.
	locations := locations collect: [:each |  each translateBy: self innerBounds origin].
	
	
	(logs at: 1) position: (locations at: 1).
	
	logs size = 2 ifTrue: [	(logs at: 2) position: (locations at: 3) ].
	
	logs size = 3 ifTrue: [	(logs at: 2) position: (locations at: 2).
							(logs at: 3) position: (locations at: 4) ].
	
	logs size = 4 ifTrue: [	(logs at: 2) position: (locations at: 2).
							(logs at: 3) position: (locations at: 3).
							(logs at: 4) position: (locations at: 4) ].
	! !

!TurnLogArea methodsFor: 'drawing' stamp: 'MichaelOSullivan 7/21/2012 11:04'!
setLogSizes: aRect
	logs do: [:each | each extent: aRect extent].! !

RectangleMorph subclass: #CardMorph
	instanceVariableNames: 'card label'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!CardMorph methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/15/2012 16:50'!
cardClass
	^card! !

!CardMorph methodsFor: 'printing' stamp: 'MichaelOSullivan 7/18/2012 20:59'!
printOn: aStream
	aStream nextPutAll: card cardName;
			  nextPutAll: '($';
			  print: card cost;
			  nextPutAll: ')'
			  
			  ! !

!CardMorph methodsFor: 'initialize' stamp: 'MichaelOSullivan 7/18/2012 22:48'!
setCardType: aCardClass board: aBoard
	| cost remaining |
	super initialize.
	self width: 80.
	self height: 100.
	card := aCardClass.
	self color: card color.
	label := (StringMorph new contents: card cardName).
	label position: (self innerBounds origin) + (5 @ 5).
	self addMorph: label.
	cost := (StringMorph new contents: '$', card cost asString).
	cost position: (self innerBounds origin) + ((self width- (1.5 * cost width)) @ (self height - cost height - 5)).
	self addMorph: cost.
	remaining := StringMorph new contents: ((aBoard cardsRemaining: aCardClass) asString).
	remaining position: (self innerBounds origin translateBy: 5 @ (self height - remaining height - 5)).
	self addMorph: remaining.! !

!CardMorph methodsFor: 'initialize' stamp: 'MichaelOSullivan 7/20/2012 13:34'!
setCardType: aCardClass count: aNumber
	| cost remaining |
	super initialize.
	self width: 80.
	self height: 100.
	card := aCardClass.
	self color: card color.
	label := (StringMorph new contents: card cardName).
	label position: (self innerBounds origin) + (5 @ 5).
	self addMorph: label.
	cost := (StringMorph new contents: '$', card cost asString).
	cost position: (self innerBounds origin) + ((self width- (1.5 * cost width)) @ (self height - cost height - 5)).
	self addMorph: cost.
	remaining := StringMorph new contents: aNumber asString.
	remaining position: (self innerBounds origin translateBy: 5 @ (self height - remaining height - 5)).
	self addMorph: remaining.! !

TestCase subclass: #CoreTest
	instanceVariableNames: 'board deck1 deck2 player1 player2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Test'!

!CoreTest methodsFor: 'private' stamp: 'JohnParker 7/18/2012 12:12'!
setupGame
	board := Board new.
	board initialize.
	board decks: 2.
	deck1 := board deck: 1.
	deck2 := board deck: 2.

	player1 := TestPlayer new.
	board addObserver: player1 at: 1.
	board addObserver: player1.
	player1 board: board deck: 1.
	player2 := TestPlayer new.
	board addObserver: player2 at: 2.
	board addObserver: player2.
	player2 board: board deck: 2.
	
	board startGame.! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:34'!
testActionCommand
	self setupGame.
	
	"Send an action command to the wrong deck.  Make sure it returns an error."
	board execute: (ActionCommand deck: 2 card: Smithy).
	self assert: player2 errorCommand isNil not.
	self assert: player1 errorCommand isNil.
	player2 clearError.
	
	"Send an action command on a non action card.  Make sure it returns an error."
	board execute: (ActionCommand deck: 1 card: Copper).
	self assert: player1 errorCommand isNil not.
	self assert: player2 errorCommand isNil.
	player1 clearError.
	
	"Send an ActionCommand to deck1 with a smithy.  Make sure that the card is played to the play area, 3 more cards are drawn by the smithy, and the phase is advanced to Treasure"
	deck1 z_addCardToHand: Smithy.
	board execute: (ActionCommand deck: 1 card: Smithy).
	self assert: player1 errorCommand isNil.
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 8.
	self assert: (deck1 play at: 1) class = Smithy.
	self assert: deck1 phase = #Treasure.
	
	"Send an action command after the action phase is complete.  Make sure it returns an error."
	deck1 z_addCardToHand: Smithy.
	board execute: (ActionCommand deck: 1 card: Smithy).
	self assert: player1 errorCommand isNil not.
	self assert: player2 errorCommand isNil.
	player1 clearError.
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:01'!
testAdventurer
	|treasure|
	self setupGame.
	
	self assert: Adventurer cost = 6.
	
	"Send an ActionCommand to deck1 with a smithy.  Make sure that the card is played to the play area, 3 more cards are drawn by the smithy, and the phase is advanced to Treasure"
	treasure := (deck1 hand select: [:each | each class inheritsFrom: TreasureCard]) size.
	deck1 z_addCardToHand: Adventurer.
	board execute: (ActionCommand deck: 1 card: Adventurer).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 7.
	self assert: (deck1 hand select: [:each | each class inheritsFrom: TreasureCard]) size = (treasure + 2).
	self assert: (deck1 play at: 1) class = Adventurer.
	self assert: deck1 phase = #Treasure.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:02'!
testBureaucrat
	self setupGame.
	
	self assert: Bureaucrat cost = 4.
	
	"Play a Bureaucrat.  Make sure that a silver is added to the top of the draw and that other players either placed a victory card on top of the draw or have no victory cards."
	deck1 z_addCardToHand: Bureaucrat.
	board execute: (ActionCommand deck: 1 card: Bureaucrat).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: deck1 z_draw last class = Silver.
	self assert: (((deck2 hand size = 4) and: (deck2 z_draw last class = Estate)) or: ((deck2 hand select: [:each | each class = Copper]) size = 5)).
	self assert: (deck1 play at: 1) class = Bureaucrat.
	self assert: deck1 phase = #Treasure.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/22/2012 21:52'!
testBuyCommand
	self setupGame.
	board addCard: Adventurer num: 10.
	
	"Make sure the phase of each deck is correct.  Deck 1 should be at Action since that is the first phase of a turn.  Deck 2 should be at None because it is not Deck 2s turn."
	self assert: deck1 phase = #Action.
	self assert: deck2 phase = #None.
	
	"Send a buy command to the correct deck.  Make sure it successfully adds the card to the discard and ends the turn"
	board execute: (TreasureCommand deck: 1 card: Copper).
	board execute: (TreasureCommand deck: 1 card: Copper).
	self assert: deck1 money = 2.
	deck1 addBuys: 1.
	board execute: (BuyCommand deck: 1 card: Estate).
	self assert: player1 errorCommand isNil.
	self assert: deck1 z_discard size = 1.
	self assert: deck1 money = 0.
	self assert: (deck1 z_discard at: 1) class = Estate.
	board execute: (BuyCommand deck: 1 card: Copper).
	self assert: deck1 z_discard size = 7.
	self assert: deck1 phase = #None.
	self assert: deck2 = board currentDeck.
	
	"Send a buy command to the wrong deck.  Make sure it returns an error."
	board execute: (BuyCommand deck: 1 card: Copper).
	self assert: player1 errorCommand isNil not.
	self assert: player2 errorCommand isNil.
	player1 clearError.
	
	"Send a buy command to the new correct deck.  Make sure it successfully adds the card to the discard and ends the turn"
	board execute: (BuyCommand deck: 2 card: Copper).
	self assert: player2 errorCommand isNil.
	self assert: deck2 z_discard size = 6.
	self assert: (deck1 z_discard at: 2) class = Copper.
	self assert: deck2 phase = #None.
	self assert: deck1 = board currentDeck.
	
	board execute: (BuyCommand deck: 1 card: Adventurer).
	self assert: player1 errorCommand isNil not.
	self assert: player2 errorCommand isNil.
	player1 clearError.
	self assert: deck1 z_discard size = 7.
	self assert: deck1 phase = #Action.
	self assert: deck1 = board currentDeck.! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:02'!
testCellar
	| victory | 
	self setupGame.
	
	self assert: Cellar cost = 2.
	
	"Play a Cellar.  We are using a default player who will discard all of his victory cards, so make sure he does so and that the correct number of cards are drawn to replace them."
	victory := (deck1 hand select: [:each | each class inheritsFrom: VictoryCard]) size.
	deck1 z_addCardToHand: Cellar.
	board execute: (ActionCommand deck: 1 card: Cellar).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: (deck1 z_discard size = victory).
	self assert: ((1 to: victory) select: [:n | (deck1 z_discard at: n) class = Estate]) size = victory.
	self assert: (deck1 play at: 1) class = Cellar.
	self assert: deck1 phase = #Action.
	self assert: deck1 actions = 1.
	self assert: deck1 z_resolving = false.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/20/2012 12:12'!
testChancellor
	| lowCards | 
	self setupGame.
	
	self assert: Chancellor cost = 3.
	
	"Play a Chancellor.  Make sure we gain $2 and the draw is cycled into the discard"
	deck1 z_addCardToHand: Chancellor.
	board execute: (ActionCommand deck: 1 card: Chancellor).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: deck1 z_draw size = 0.
	self assert: deck1 z_discard size = 5.
	self assert: deck1 money = 2.
	self assert: (deck1 play at: 1) class = Chancellor.
	self assert: deck1 phase = #Treasure.
	self assert: deck1 actions = 0.
	self assert: deck1 z_resolving = false.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/20/2012 12:10'!
testChapel
	| lowCards | 
	self setupGame.
	
	self assert: Chapel cost = 2.
	
	"Play a Cellar.  We are using a default player who will discard all of his victory cards, so make sure he does so and that the correct number of cards are drawn to replace them."
	lowCards := (deck1 hand select: [:each | (each class = Copper) or: (each class = Estate)]) size min: 4.
	deck1 z_addCardToHand: Chapel.
	board execute: (ActionCommand deck: 1 card: Chapel).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = (5 - lowCards).
	self assert: (board z_trash size = lowCards).
	self assert: (deck1 play at: 1) class = Chapel.
	self assert: deck1 phase = #Treasure.
	self assert: deck1 actions = 0.
	self assert: deck1 z_resolving = false.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/20/2012 16:30'!
testCouncilRoom
	self setupGame.
	
	self assert: CouncilRoom cost = 5.
	
	"Play a Council Room. Make sure 4 cards are drawn, 1 buy is added, and other players draw a card."

	deck1 z_addCardToHand: CouncilRoom.	

	board execute: (ActionCommand deck: 1 card: CouncilRoom).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 9.
	self assert: deck1 buys = 2.
	self assert: deck1 actions = 0.
	self assert: (deck1 play at: 1) class = CouncilRoom.
	self assert: deck1 phase = #Treasure.
	self assert: deck2 hand size = 6.

	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:35'!
testEndTurnCommand
	self setupGame.
	
	"Send an end turn command to the wrong deck.  Make sure it returns an error."
	board execute: (EndTurnCommand deck: 2).
	self assert: player2 errorCommand isNil not.
	self assert: player1 errorCommand isNil.
	player2 clearError.
	
	"Send an end turn command to deck1."
	"Before: draw(5) discard(0) hand(5) play(0)"
	"After:    draw(0) discard(5) hand(5) play(0)"
	board execute: (EndTurnCommand deck: 1).
	self assert: player2 errorCommand isNil.
	self assert: deck1 play size = 0.
	self assert: deck1 hand size = 5.
	self assert: deck1 z_discard size = 5.
	self assert: deck1 z_draw size = 0.
	self assert: deck1 phase = #None.
	
	"Send an end turn command to the wrong deck.  Make sure it returns an error."
	board execute: (EndTurnCommand deck: 1).
	self assert: player1 errorCommand isNil not.
	self assert: player2 errorCommand isNil.
	player1 clearError.
	
	"Send an end turn command to deck2."
	"Before: draw(5) discard(0) hand(5) play(0)"
	"After:    draw(0) discard(5) hand(5) play(0)"
	board execute: (EndTurnCommand deck: 2).
	self assert: player2 errorCommand isNil.
	self assert: deck2 play size = 0.
	self assert: deck2 hand size = 5.
	self assert: deck2 z_discard size = 5.
	self assert: deck2 z_draw size = 0.
	self assert: deck2 phase = #None.
	
	"Send an end turn command to the wrong deck.  Make sure it returns an error."
	board execute: (EndTurnCommand deck: 2).
	self assert: player2 errorCommand isNil not.
	self assert: player1 errorCommand isNil.
	player2 clearError.
	
	"Send a play treasure command to deck1."
	"Before: draw(0) discard(5) hand(5) play(0)"
	"After:    draw(0) discard(5) hand(4) play(1)"
	board execute: (TreasureCommand deck: 1 card: Copper).
	self assert: player1 errorCommand isNil.
	
	"Send a buy command to deck1.  This also ends the turn because the deck is out of buys"
	"Before:     draw(0) discard(5) hand(4) play(1)"
	"After buy: draw(0) discard(6) hand(4) play(1)"
	"After end: draw(6) discard(0) hand(5) play(0)"
	board execute: (BuyCommand deck: 1 card: Copper).
	self assert: player1 errorCommand isNil.
	self assert: deck1 play size = 0.
	self assert: deck1 hand size = 5.
	self assert: deck1 z_discard size = 0.
	self assert: deck1 z_draw size = 6.
	self assert: deck1 phase = #None.
	
	"Send an end turn command to the wrong deck.  Make sure it returns an error."
	board execute: (EndTurnCommand deck: 1).
	self assert: player1 errorCommand isNil not.
	self assert: player2 errorCommand isNil.
	player1 clearError.
	
	"Send an end turn command to deck2."
	"Before: draw(0) discard(5) hand(5) play(0)"
	"After:   draw(5) discard(0) hand(5) play(0)"
	board execute: (EndTurnCommand deck: 2).
	self assert: player2 errorCommand isNil.
	self assert: deck2 play size = 0.
	self assert: deck2 hand size = 5.
	self assert: deck2 z_discard size = 0.
	self assert: deck2 z_draw size = 5.
	self assert: deck2 phase = #None.! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/20/2012 12:27'!
testFeast
	self setupGame.
	
	self assert: Feast cost = 4.
	
	"Play a Feast.  The test player will choose to gain a silver.  Make sure he does so. Also ensure that the feast gets trashed."
	deck1 z_addCardToHand: Feast.
	board execute: (ActionCommand deck: 1 card: Feast).
	self assert: deck1 play size = 0.
	self assert: board trashTop class = Feast.
	self assert: deck1 hand size = 5.
	self assert: (deck1 z_discard size = 1).
	self assert: deck1 z_discard last class = Silver.
	self assert: deck1 phase = #Treasure.
	self assert: deck1 z_resolving = false.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/20/2012 18:17'!
testFestival
	self setupGame.
	
	self assert: Festival cost = 5.
	
	"Play a Festival. Make sure 2 actions, 1 buy, and 2 money are added"

	deck1 z_addCardToHand: Festival.	

	board execute: (ActionCommand deck: 1 card: Festival).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: deck1 actions = 2.
	self assert: deck1 buys = 2.
	self assert: deck1 money = 2.
	self assert: (deck1 play at: 1) class = Festival.
	self assert: deck1 phase = #Action.

	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/20/2012 12:40'!
testGameEnd
	self testSetup.
	board  addCard: Gardens num: 8.
	
	1 to: 7 do: [:a |
		4 timesRepeat: [deck1 z_addCardToHand: Gold].
		4 timesRepeat: [board execute: (TreasureCommand deck: 1 card: Gold)].
		
		deck1 addBuys: 1.
		board execute: (BuyCommand deck: 1 card: Province).
		board execute: (BuyCommand deck: 1 card: Gardens).
		
		board execute: (EndTurnCommand deck: 2).
		
		self assert: board gameOver not.
		self should: [deck1 victoryPoints] raise: Error.
	].
	
	1 to: 3 do: [:b | deck1 z_addCardToHand: Gold].
	1 to: 3 do: [:b |board execute: (TreasureCommand deck: 1 card: Gold)].
	board execute: (BuyCommand deck: 1 card: Province).
	
	self assert: board gameOver.
	self assert: board winner = 1.
	self assert: deck1 victoryPoints = ((6*8)+3+(7*(deck1 allCards size / 10) floor)).
	self assert: deck2 victoryPoints = 3.! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/20/2012 16:34'!
testLaboratory
	self setupGame.
	
	self assert: Laboratory cost = 5.
	
	"Play a Laboratory. Make sure 2 cards are drawn and 1 action is added."

	deck1 z_addCardToHand: Laboratory.	

	board execute: (ActionCommand deck: 1 card: Laboratory).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 7.
	self assert: deck1 actions = 1.
	self assert: (deck1 play at: 1) class = Laboratory.
	self assert: deck1 phase = #Action.

	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/20/2012 13:04'!
testMilitia
	self setupGame.
	
	self assert: Militia cost = 4.
	
	"Play a Bureaucrat.  Make sure that a silver is added to the top of the draw and that other players either placed a victory card on top of the draw or have no victory cards."
	deck1 z_addCardToHand: Militia.
	board execute: (ActionCommand deck: 1 card: Militia).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: deck1 money = 2.
	self assert: deck2 hand size = 3.
	self assert: deck2 z_discard size = 2.
	self assert: (deck1 play at: 1) class = Militia.
	self assert: deck1 phase = #Treasure.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/18/2012 12:09'!
testMine
	| copper | 
	self setupGame.
	
	self assert: Mine cost = 5.
	
	"Play a Mine.  We are using a default player who will trash copper for silver, so make sure he does so."
	copper := (deck1 hand select: [:each | each class = Copper]) size.
	deck1 z_addCardToHand: Mine.
	board execute: (ActionCommand deck: 1 card: Mine).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: board trashTop class = Copper.
	self assert: (deck1 z_discard size = 0).
	self assert: (deck1 hand select: [:each | each class = Copper]) size = (copper - 1).
	self assert: (deck1 handContains: Silver).
	self assert: (deck1 play at: 1) class = Mine.
	self assert: deck1 phase = #Treasure.
	self assert: deck1 z_resolving = false.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/18/2012 20:14'!
testMoat
	self setupGame.
	
	self assert: Moat cost = 2.
	
	"Play a Bureaucrat.  Make sure that nothing happens because player 2 reveals the moat."
	deck1 z_addCardToHand: Bureaucrat.
	deck2 z_addCardToHand: Moat.
	deck2 z_addCardToHand: Estate.
	board execute: (ActionCommand deck: 1 card: Bureaucrat).
	self assert: deck2 hand size = 7.
	
	self setupGame.
	
	"Play a Moat. Make sure 2 cards are drawn"
	deck1 z_addCardToHand: Moat.	

	board execute: (ActionCommand deck: 1 card: Moat).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 7.
	self assert: deck1 actions = 0.
	self assert: (deck1 play at: 1) class = Moat.
	self assert: deck1 phase = #Treasure.

	
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/20/2012 13:15'!
testMoneylender
	self setupGame.
	
	self assert: Moneylender cost = 4.
	
	"Play a Moneylender with copper in hand.  Make sure the copper is trashed and money is +3"

	deck1 z_addCardToHand: Moneylender.	

	board execute: (ActionCommand deck: 1 card: Moneylender).
	self assert: deck1 play size = 1.
	self assert: board trashTop class = Copper.
	self assert: deck1 hand size = 4.
	self assert: deck1 money = 3.
	self assert: deck1 actions = 0.
	self assert: (deck1 play at: 1) class = Moneylender.
	self assert: deck1 phase = #Treasure.
	
	
	"Play a Moneylender with no copper in hand.  Make sure nothing happens."
	self setupGame.
	deck1 z_areas at: #hand put: CardArea new.
	deck1 z_addCardToHand: Moneylender.
	4 timesRepeat: [deck1 z_addCardToHand: Estate].	
	
	board execute: (ActionCommand deck: 1 card: Moneylender).
	self assert: deck1 play size = 1.
	self assert: board z_trash size = 0.
	self assert: deck1 hand size = 4.
	self assert: deck1 money = 0.
	self assert: deck1 actions = 0.
	self assert: (deck1 play at: 1) class = Moneylender.
	self assert: deck1 phase = #Treasure.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/20/2012 16:19'!
testRemodel
	|estates|
	self setupGame.
	
	self assert: Remodel cost = 4.
	
	"Play a Remodel.  We are using a default player who will trash an estate for silver, so make sure he does so."
	deck1 z_addCardToHand: Estate.
	estates := (deck1 hand select: [:each | each class = Estate]) size.
	deck1 z_addCardToHand: Remodel.
	board execute: (ActionCommand deck: 1 card: Remodel).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: board trashTop class = Estate.
	self assert: (deck1 z_discard size = 1).
	self assert: (deck1 z_discard last class = Silver).
	self assert: (deck1 hand select: [:each | each class = Estate]) size = (estates - 1).
	self assert: (deck1 play at: 1) class = Remodel.
	self assert: deck1 phase = #Treasure.
	self assert: deck1 z_resolving = false.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 13:05'!
testSetup
	| actions cards temp|
	self setupGame.

	self assert: (deck1 z_draw select: [:each | each class = Copper]) size + (deck1 hand select: [:each | each class = Copper]) size= 7.
	self assert: (deck1 z_draw select: [:each | each class = Estate]) size + (deck1 hand select: [:each | each class = Estate]) size= 3.
	
	self assert: (deck2 z_draw select: [:each | each class = Copper]) size + (deck2 hand select: [:each | each class = Copper]) size= 7.
	self assert: (deck2 z_draw select: [:each | each class = Estate]) size + (deck2 hand select: [:each | each class = Estate]) size= 3.
	
	self assert: ((board cardsRemaining: Copper) = (60 - (7*2))).
	self assert: ((board cardsRemaining: Estate) = 8).
	
	actions := {Adventurer. Village. Smithy. Cellar. Mine.}.
	cards := {Copper. Silver. Gold. Estate. Duchy. Province. Curse. Adventurer. Village. Smithy. Cellar. Mine.}.
	
	actions do: [:each | board addCard: each num: 10].
	
	temp := board cardsAvailable.
	self assert: (temp includesAll: cards).
	self assert: (cards includesAll: temp).
	
	temp := board actionsAvailable.
	self assert: (temp includesAll: actions).
	self assert: (actions includesAll: temp).
	actions do: [:each | board cardsRemaining: each = 10].
	
	self assert: (board cardStackEmpty: Village) not.
	10 timesRepeat: [board takeCard: Village deck: 1].
	self assert: (board cardStackEmpty: Village).
	
! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:04'!
testSmithy
	self setupGame.
	
	self assert: Smithy cost = 4.
	
	"Play a Smithy. Make sure 3 cards are drawn"

	deck1 z_addCardToHand: Smithy.	

	board execute: (ActionCommand deck: 1 card: Smithy).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 8.
	self assert: deck1 actions = 0.
	self assert: (deck1 play at: 1) class = Smithy.
	self assert: deck1 phase = #Treasure.

	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/19/2012 20:42'!
testSpy
	self setupGame.
	
	self assert: Spy cost = 4.
	
	"Play a Spy.  Make sure that the player draws 2 cards and other players gain a curse."
	deck1 z_draw addLast: Estate basicNew.
	deck1 z_draw addLast: Estate basicNew.
	deck2 z_draw addLast: Copper basicNew.
	deck1 z_addCardToHand: Spy.
	board execute: (ActionCommand deck: 1 card: Spy).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 6.
	self assert: deck1 actions = 1.
	self assert: (deck1 play at: 1) class = Spy.
	self assert: deck1 phase = #Action.
	self assert: deck1 z_resolving = false.
	self assert: deck1 z_draw size = 5.
	self assert: deck2 z_draw size = 5.
	self assert: deck1 z_discard last class = Estate.
	self assert: deck2 z_discard last class = Copper.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/22/2012 21:49'!
testThief
	self setupGame.
	
	self assert: Thief cost = 4.
	
	"Play a Spy.  Make sure that player 2 reveals the top 2 cards of his deck, and that player 1 takes 1 of them and the other is discarded."
	deck2 z_draw addLast: Copper basicNew.
	deck2 z_draw addLast: Estate basicNew.
	deck1 z_addCardToHand: Thief.
	board execute: (ActionCommand deck: 1 card: Thief).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: deck1 actions = 0.
	self assert: (deck1 play at: 1) class = Thief.
	self assert: deck1 phase = #Treasure.
	self assert: deck1 z_resolving = false.
	self assert: deck1 z_draw size = 5.
	self assert: deck1 z_discard size = 1.
	self assert: deck2 z_draw size = 5.
	self assert: deck2 z_discard size = 1.
	self assert: deck2 z_discard last class = Estate.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/22/2012 21:08'!
testThroneRoom
	self setupGame.
	
	self assert: ThroneRoom cost = 4.
	
	"Play a Throne Room on a Village.  Make sure the village gets played twice."
	deck1 z_addCardToHand: ThroneRoom.
	deck1 z_addCardToHand: Village.
	board execute: (ActionCommand deck: 1 card: ThroneRoom).
	self assert: deck1 play size = 2.
	self assert: deck1 hand size = 7.
	self assert: deck1 actions = 4.
	self assert: deck1 phase = #Action.
	self assert: deck1 z_resolving = false.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:37'!
testTreasureCommand
	self setupGame.
	
	"Send a TreasureCommand to deck1.  Make sure that the card is played to the play area and that the money is increased as expected."
	board execute: (TreasureCommand deck: 1 card: Copper).
	self assert: player1 errorCommand isNil.
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 4.
	self assert: deck1 money = 1.
	self assert: (deck1 play at: 1) class = Copper.
	self assert: deck1 phase = #Treasure.
	
	"Send a treasure command to the wrong deck.  Make sure it returns an error."
	board execute: (TreasureCommand deck: 2 card: Copper).
	self assert: player2 errorCommand isNil not.
	self assert: player1 errorCommand isNil.
	player2 clearError.
	
	"Send a TreasureCommand to deck1.  Make sure that the card is played to the play area and that the money is increased as expected."
	board execute: (TreasureCommand deck: 1 card: Copper).
	self assert: player1 errorCommand isNil.
	self assert: deck1 play size = 2.
	self assert: deck1 hand size = 3.
	self assert: deck1 money = 2.
	self assert: (deck1 play at: 1) class = Copper.
	self assert: deck1 phase = #Treasure.
	
	"Try to buy an estate.  This costs 2, which we have due to having played 2 coppers"
	board execute: (BuyCommand deck: 1 card: Estate).
	self assert: player1 errorCommand isNil.
	self assert: deck1 z_discard size = 6.
	self assert: (deck1 z_discard at: 1) class = Estate.
	self assert: deck1 phase = #None.
	self assert: deck2 = board currentDeck.! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:05'!
testVillage
	self setupGame.
	
	self assert: Village cost = 3.
	
	"Play a Village. Make sure a card was drawn, the number of actions is now 2, and the phase is still #Action"
	deck1 z_addCardToHand: Village.
	deck1 z_addCardToHand: Smithy.
	deck1 z_addCardToHand: Smithy.
	board execute: (ActionCommand deck: 1 card: Village).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 8.
	self assert: deck1 actions = 2.
	self assert: (deck1 play at: 1) class = Village.
	self assert: deck1 phase = #Action.
	
	"Make sure we can now play two more cards."
	board execute: (ActionCommand deck: 1 card: Smithy).
	self assert: deck1 play size = 2.
	self assert: deck1 hand size = 10.
	self assert: deck1 actions = 1.
	self assert: (deck1 play at: 2) class = Smithy.
	self assert: deck1 phase = #Action.
	
	board execute: (ActionCommand deck: 1 card: Smithy).
	self assert: deck1 play size = 3.
	self assert: deck1 hand size = 10.
	self assert: deck1 actions = 0.
	self assert: (deck1 play at: 3) class = Smithy.
	self assert: deck1 phase = #Treasure.
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/18/2012 20:26'!
testWitch
	self setupGame.
	
	self assert: Witch cost = 5.
	
	"Play a Witch.  Make sure that the player draws 2 cards and other players gain a curse."
	deck1 z_addCardToHand: Witch.
	board execute: (ActionCommand deck: 1 card: Witch).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 7.
	self assert: (deck1 play at: 1) class = Witch.
	self assert: deck1 phase = #Treasure.
	self assert: (deck2 z_discard last class = Curse).
	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/20/2012 12:16'!
testWoodcutter
	self setupGame.
	
	self assert: Woodcutter cost = 3.
	
	"Play a Smithy. Make sure 3 cards are drawn"

	deck1 z_addCardToHand: Woodcutter.	

	board execute: (ActionCommand deck: 1 card: Woodcutter).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: deck1 actions = 0.
	self assert: deck1 buys = 2.
	self assert: deck1 money = 2.
	self assert: (deck1 play at: 1) class = Woodcutter.
	self assert: deck1 phase = #Treasure.

	
	! !

!CoreTest methodsFor: 'testing' stamp: 'JohnParker 7/20/2012 12:21'!
testWorkshop
	self setupGame.
	
	self assert: Workshop cost = 3.
	
	"Play a Workshop.  The test player will choose to gain a silver.  Make sure he does so."
	deck1 z_addCardToHand: Workshop.
	board execute: (ActionCommand deck: 1 card: Workshop).
	self assert: deck1 play size = 1.
	self assert: deck1 hand size = 5.
	self assert: (deck1 z_discard size = 1).
	self assert: deck1 z_discard last class = Silver.
	self assert: (deck1 play at: 1) class = Workshop.
	self assert: deck1 phase = #Treasure.
	self assert: deck1 z_resolving = false.
	
	! !

TestCase subclass: #DisplayTest
	instanceVariableNames: 'board deck1 deck2 player1 player2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Test'!

!DisplayTest methodsFor: 'private' stamp: 'MichaelOSullivan 7/22/2012 14:24'!
setupGame
	board := Board new.
	board initialize.! !

!DisplayTest methodsFor: 'testing' stamp: 'MichaelOSullivan 7/22/2012 14:33'!
test2PlayerGameTable
	self setupGame.
	board decks: 2.
	board  addCard: Adventurer num: 10.
	board  addCard: Bureaucrat num: 10.
	board  addCard: Cellar num: 10.
	board  addCard: Mine num: 10.
	board  addCard: Smithy num: 10.
	board  addCard: Village num: 10.
	board  addCard: Moat num: 10.
	board  addCard: Witch num: 10.
	board  addCard: Chancellor num: 10.
	board  addCard: Chapel num: 10.
	
	(GameTable new initializeWith: board) openInWorld.! !

!DisplayTest methodsFor: 'testing' stamp: 'MichaelOSullivan 7/22/2012 14:31'!
test3PlayerGameTable
	self setupGame.
	board decks: 3.
	board  addCard: Remodel num: 10.
	board  addCard: ThroneRoom num: 10.
	board  addCard: Woodcutter num: 10.
	board  addCard: Workshop num: 10.
	board  addCard: Smithy num: 10.
	board  addCard: Village num: 10.
	board  addCard: Moat num: 10.
	board  addCard: Spy num: 10.
	board  addCard: Feast num: 10.
	board  addCard: Festival num: 10.
	
	(GameTable new initializeWith: board) openInWorld.! !

!DisplayTest methodsFor: 'testing' stamp: 'MichaelOSullivan 7/22/2012 14:33'!
test4PlayerGameTable
	self setupGame.
	board decks: 4.
	board  addCard: Chancellor num: 10.
	board  addCard: Chapel num: 10.
	board  addCard: CouncilRoom num: 10.
	board  addCard: Festival num: 10.
	board  addCard: Gardens num: 10.
	board  addCard: Laboratory num: 10.
	board  addCard: Library num: 10.
	board  addCard: Militia num: 10.
	board  addCard: Moneylender num: 10.
	board  addCard: Spy num: 10.
	
	(GameTable new initializeWith: board) openInWorld.! !

Object subclass: #Board
	instanceVariableNames: 'cards decks trash currentDeck gameOver globalEvents playerEvents'
	classVariableNames: 'UniqueInstance'
	poolDictionaries: ''
	category: 'Dominion'!

!Board commentStamp: 'JohnParker 7/17/2012 22:04' prior: 0!
A Board is a singleton representing the overall game state of Dominion.

Instance Variables
	cards:			Bag
	currentDeck:	Number
	decks:			OrderedCollection
	trash: 			OrderedCollection 
	gameOver:		Boolean
	globalEvents	Array
	playerEvents	Array

cards
	- a Bag containing the card classes available and the number left of each class

currentDeck
	- the deck whose turn it currently is.

decks
	- the list of decks in the game.  

trash
	- a list of cards that have been trashed so far.

gameOver 
	- indicates whether the game is over.

globalEvents
	- the events that are sent to all players when triggered.
	Signatures:
		deck: aNumber cardMovedGlobal: aCardClass from: area1 to: area2
		deck: aNumber took: aCardClass
		deck: aNumber trashed: aCardClass
		turnStarted: aNumber
		deckShuffled: aNumber	
		
playerEvents
	- the events that are sent to a specific player when triggered.  These events carry information that is private to a given player.
	Signatures
		deck: aNumber cardMovedPlayer: aCardClass from: area1 to: area2
		resolveAction: aCardClass
		respondToCard: UNDEFINED
		command: aCommand error: aString!

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 12:38'!
actionsAvailable
	^cards asSet select: [:each | each inheritsFrom: ActionCard].! !

!Board methodsFor: 'setup' stamp: 'JohnParker 7/3/2012 20:45'!
addCard: aCardClass num: aNumber
	cards add: aCardClass withOccurrences: aNumber! !

!Board methodsFor: 'setup' stamp: 'JohnParker 7/18/2012 12:08'!
addObserver: anObserver
	globalEvents do: [:event | self when: event send: event to: anObserver].
	! !

!Board methodsFor: 'setup' stamp: 'JohnParker 7/17/2012 12:18'!
addObserver: anObserver at: deckNum
	playerEvents do: [:event | (decks at: deckNum) when: event send: event to: anObserver].
	! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/3/2012 21:38'!
cardStackEmpty: aCardClass
	^(cards includes: aCardClass) not! !

!Board methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/15/2012 13:59'!
cards
	^cards! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 12:36'!
cardsAvailable
	^cards asSet! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 02:27'!
cardsRemaining: aCardClass
	^cards occurrencesOf: aCardClass! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 00:56'!
currentDeck
	^decks at: currentDeck! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 00:29'!
deck: aNumber
	^decks at: aNumber ifAbsent: [^nil]! !

!Board methodsFor: 'setup' stamp: 'JohnParker 7/19/2012 19:52'!
decks: aNumber
	| victoryNumber deck|
	cards := Bag new.
	
	"Add the standard cards to the game"
	cards add: Copper withOccurrences: 60.
	cards add: Silver withOccurrences: 40.
	cards add: Gold withOccurrences: 30.
	
	victoryNumber := (aNumber = 2) ifTrue: [8] ifFalse: [12].
	cards add: Estate withOccurrences: (victoryNumber + (3*aNumber)).
	cards add: Duchy withOccurrences: victoryNumber.
	cards add: Province withOccurrences: victoryNumber.
	cards add: Curse withOccurrences: (10 * (aNumber - 1)).
	
	1 to: aNumber do: [:num | 
		deck := Deck number: num board: self.
		decks add: deck.
		globalEvents do: [:event | deck when: event send: event to: self].
	].
	currentDeck := 1.
	! !

!Board methodsFor: 'events' stamp: 'JohnParker 7/13/2012 13:30'!
doesNotUnderstand: aMessage
	(globalEvents includes: aMessage selector) ifFalse: [super doesNotUnderstand: aMessage].
	self triggerEvent: aMessage selector withArguments: aMessage arguments
	! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 21:28'!
events
	|list|
	list := OrderedCollection new.
	list addAll: globalEvents.
	list addAll: playerEvents.
	^list.! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/18/2012 12:01'!
execute: aCommand
	aCommand do: self.! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:39'!
gameOver
	^gameOver.! !

!Board methodsFor: 'initialize-release' stamp: 'JohnParker 7/18/2012 21:26'!
initialize
	decks := OrderedCollection new.
	currentDeck := 1.
	cards := Bag new.
	trash := OrderedCollection new.
	gameOver := false.
	globalEvents := {#deck:cardMovedGlobal:from:to:. #turnStarted:. #deckShuffled. #deck:took:. #deck:trashed:.}.
	playerEvents := {#deck:cardMovedPlayer:from:to:. #resolveAction:. #resolveAction:deck:. #attack:. #command:error:.}.! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/5/2012 21:07'!
nextDeck
	(cards includes: Province) ifFalse: [
		gameOver := true.
		currentDeck := 0.
	] ifTrue: [
		currentDeck := (currentDeck rem: decks size) + 1.
		(decks at: currentDeck) startTurn.
	]! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/18/2012 20:39'!
numDecks
	^decks size.! !

!Board methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/21/2012 08:36'!
numberOfPlayers
	^decks size! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/4/2012 23:43'!
otherDecks
	^((1 to: decks size) reject: [:num | num = currentDeck]) collect: [:num | decks at: num].! !

!Board methodsFor: 'setup' stamp: 'JohnParker 7/4/2012 13:49'!
startGame
	(decks at: 1) startTurn.! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/13/2012 13:01'!
takeCard: aCardClass deck: aNumber
	cards remove: aCardClass ifAbsent: [^nil].
	self triggerEvent: #deck:took: withArguments: {aNumber. aCardClass}.
	^aCardClass basicNew.! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/22/2012 21:30'!
takeTrashTop
	^trash removeLast.! !

!Board methodsFor: 'game' stamp: 'JohnParker 7/13/2012 13:01'!
trashCard: aCard deck: deckNumber
	trash addLast: aCard.
	self triggerEvent: #deck:trashed: withArguments: {deckNumber. aCard class}.! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 12:41'!
trashTop
	^trash last.! !

!Board methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:39'!
winner
	|tempDecks|
	gameOver ifFalse: [self error: 'The game is not over so there is no winner yet.'].
	tempDecks := OrderedCollection new.
	tempDecks addAll: decks.
	tempDecks sort: [:first :second | first victoryPoints > second victoryPoints].
	^decks indexOf: (tempDecks at: 1).! !

!Board methodsFor: 'testing' stamp: 'JohnParker 7/20/2012 11:56'!
z_trash
	^trash! !

Object subclass: #Card
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!Card commentStamp: 'JohnParker 7/3/2012 19:31' prior: 0!
A Card is an abstract class at the top of the card hierarchy.  It represents a generic card in Dominion.!

Card subclass: #ActionCard
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!ActionCard commentStamp: 'JohnParker 7/4/2012 14:49' prior: 0!
An ActionCard is an abstract class representing a Dominion action card.  !

!ActionCard class methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/14/2012 08:27'!
color
	^Color white! !

!ActionCard methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 13:55'!
cardName
	^'Action'! !

!ActionCard methodsFor: 'game' stamp: 'JohnParker 7/18/2012 11:57'!
resolve: argVector board: board! !

ActionCard subclass: #Adventurer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Adventurer class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 23:01'!
cardName
	^'Adventurer'! !

!Adventurer class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 22:04'!
cost
	^6! !

!Adventurer methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:18'!
playCard: board
	| numTreasure card deck|
	numTreasure := 0.
	deck := board currentDeck.
	[numTreasure < 2] whileTrue: [
		card := deck drawCardTo: #peek.
		card isNil ifTrue: [numTreasure := 2].
		
		(card class inheritsFrom: TreasureCard) ifTrue: [
			deck moveCard: card class from: #peek to: #hand.
			numTreasure := numTreasure + 1
		] ifFalse: [
			deck moveCard: card class from: #peek to: #discard.
		].
	].! !

ActionCard subclass: #AttackCard
	instanceVariableNames: 'playersResolved'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!AttackCard methodsFor: 'game' stamp: 'JohnParker 7/18/2012 12:58'!
resolveAttack: args board: board! !

AttackCard subclass: #Bureaucrat
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Bureaucrat class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 23:30'!
cardName
	^'Bureaucrat'! !

!Bureaucrat class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 23:30'!
cost
	^4! !

!Bureaucrat methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:18'!
playCard: board
	playersResolved := OrderedCollection new.
	board numDecks timesRepeat: [playersResolved add: false].
	playersResolved at: board currentDeck number put: true.
	
	board currentDeck gainCard: Silver to: #draw.
	board otherDecks do: [:each | each attack: self].! !

!Bureaucrat methodsFor: 'game' stamp: 'JohnParker 7/19/2012 20:41'!
resolveAttack: args board: board deck: deck
	|cardClass card|
	
	(playersResolved at: deck number) ifTrue: [self error: 'That attack has already been resolved.'].
	
	cardClass := args at: #bureaucratCard ifAbsent: [
		((deck hand select: [:each | each class inheritsFrom: VictoryCard]) size = 0) ifFalse: [
			self error: 'You must choose a victory card if you have one.'
		].
		deck hand do: [:each | deck moveCard: each class from: #hand to: #reveal].
		deck reveal do: [:each | deck moveCard: each class from: #reveal to: #hand].
		playersResolved at: deck number put: true.
		board currentDeck cardResolved: (playersResolved inject: true into: [:val :each | val and: each]).
		deck attack: nil.
		^self.
	].
	
	(cardClass inheritsFrom: VictoryCard) ifFalse: [self error: 'The card you choose must be a victory card.'].
	
	card := deck hand detect: [:each | each class = cardClass] ifNone: [self error: 'You must choose a card you have in your hand.'].
	
	deck moveCard: card class from: #hand to: #draw.
	playersResolved at: deck number put: true.
	board currentDeck cardResolved: (playersResolved inject: true into: [:val :each | val and: each]).
	deck attack: nil.
	
	! !

AttackCard subclass: #Militia
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Militia class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:41'!
cardName
	^'Militia'! !

!Militia class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:41'!
cost
	^4! !

!Militia methodsFor: 'game' stamp: 'JohnParker 7/20/2012 12:41'!
playCard: board
	playersResolved := OrderedCollection new.
	board numDecks timesRepeat: [playersResolved add: false].
	playersResolved at: board currentDeck number put: true.
	
	board currentDeck addMoney: 2.
	board otherDecks do: [:each | each attack: self].! !

!Militia methodsFor: 'game' stamp: 'JohnParker 7/20/2012 12:45'!
resolveAttack: args board: board deck: deck
	|discards|
	
	(playersResolved at: deck number) ifTrue: [self error: 'That attack has already been resolved.'].
	
	discards := args at: #discards.
	
	((deck hand size - discards size) = 3) ifFalse: [self error: 'You must discard down to 3 cards in your hand.'].
	
	(deck handContainsAll: discards) ifFalse: [self error: 'You can only discard the cards you have in your hand.'].
	
	discards do: [:each | deck moveCard: each from: #hand to: #discard].
	
	playersResolved at: deck number put: true.
	board currentDeck cardResolved: (playersResolved inject: true into: [:val :each | val and: each]).
	deck attack: nil.
	
	! !

AttackCard subclass: #Spy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Spy class methodsFor: 'accessing' stamp: 'JohnParker 7/18/2012 20:27'!
cardName
	^'Spy'! !

!Spy class methodsFor: 'accessing' stamp: 'JohnParker 7/18/2012 20:27'!
cost
	^4! !

!Spy methodsFor: 'game' stamp: 'JohnParker 7/19/2012 20:24'!
playCard: board
	playersResolved := OrderedCollection new.
	board numDecks timesRepeat: [playersResolved add: false].

	board currentDeck drawCardTo: #hand.
	board currentDeck addActions: 1.
	board currentDeck cardResolved: false.
	
	board otherDecks do: [:each | each attack: self].
	
	board currentDeck drawCardTo: #reveal.
	board currentDeck triggerEvent: #resolveAction:deck: withArguments: {self class. board currentDeck number}.! !

!Spy methodsFor: 'game' stamp: 'JohnParker 7/19/2012 20:37'!
resolve: args board: board
	| deckNumber discard deck|
	
	deckNumber := args at: #deck.
	discard := args at: #discard.
	
	(playersResolved at: deckNumber) ifTrue: [self error: 'That player has already been resolved.'].
	
	deck := board deck: deckNumber.
	discard ifTrue: [
		deck moveCard: deck reveal last class from: #reveal to: #discard
	] ifFalse: [
		deck moveCard: deck reveal last class from: #reveal to: #draw
	].
	
	playersResolved at: deckNumber put: true.
	deck attack: nil.
	
	board currentDeck cardResolved: (playersResolved inject: true into: [:val :each | val and: each]).! !

!Spy methodsFor: 'game' stamp: 'JohnParker 7/19/2012 20:31'!
resolveAttack: args board: board deck: deck
	(playersResolved at: deck number) ifTrue: [self error: 'That attack has already been resolved.'].
	deck drawCardTo: #reveal.
	
	board currentDeck triggerEvent: #resolveAction:deck: withArguments: {self class. deck number}.
	! !

AttackCard subclass: #Thief
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Thief class methodsFor: 'accessing' stamp: 'JohnParker 7/22/2012 21:11'!
cardName
	^'Spy'! !

!Thief class methodsFor: 'accessing' stamp: 'JohnParker 7/22/2012 21:11'!
cost
	^4! !

!Thief methodsFor: 'game' stamp: 'JohnParker 7/22/2012 21:49'!
playCard: board
	playersResolved := OrderedCollection new.
	board numDecks timesRepeat: [playersResolved add: false].
	playersResolved at: board currentDeck number put: true.
	
	board currentDeck cardResolved: false.
	
	board otherDecks do: [:each | each attack: self].! !

!Thief methodsFor: 'game' stamp: 'JohnParker 7/22/2012 21:51'!
resolve: args board: board
	| trash take deckNumber deck|
	
	deckNumber := args at: #deck.
	trash := args at: #trash.
	take := args at: #take.
	
	(playersResolved at: deckNumber) ifTrue: [self error: 'That player has already been resolved.'].
	
	deck := board deck: deckNumber.
	
	trash isNil ifTrue: [
		(deck reveal contains: [:each | each class inheritsFrom: TreasureCard]) ifTrue: [
			self error: 'You must select a treasure card to trash.'
		] ifFalse: [
			2 timesRepeat: [deck moveCard: deck reveal last from: #reveal to: #discard].
		]
	] ifFalse: [
		(deck reveal contains: [:each | each class = trash]) ifFalse: [
			self error: 'You must select a treasure card to trash that the player has revealed.'
		] ifTrue: [
			deck trashCard: trash from: #reveal.
			deck moveCard: deck reveal last class from: #reveal to: #discard.
			
			take ifTrue: [board currentDeck takeTrashTopTo: #discard].
		].
	].
	
	playersResolved at: deckNumber put: true.
	deck attack: nil.
	
	board currentDeck cardResolved: (playersResolved inject: true into: [:val :each | val and: each]).! !

!Thief methodsFor: 'game' stamp: 'JohnParker 7/22/2012 21:13'!
resolveAttack: args board: board deck: deck
	(playersResolved at: deck number) ifTrue: [self error: 'That attack has already been resolved.'].
	2 timesRepeat: [deck drawCardTo: #reveal].
	
	board currentDeck triggerEvent: #resolveAction:deck: withArguments: {self class. deck number}.
	! !

AttackCard subclass: #Witch
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Witch class methodsFor: 'accessing' stamp: 'JohnParker 7/18/2012 20:20'!
cardName
	^'Witch'! !

!Witch class methodsFor: 'accessing' stamp: 'JohnParker 7/18/2012 20:20'!
cost
	^5! !

!Witch methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:19'!
playCard: board
	playersResolved := OrderedCollection new.
	board numDecks timesRepeat: [playersResolved add: false].
	playersResolved at: board currentDeck number put: true.

	2 timesRepeat: [board currentDeck drawCardTo: #hand ].
	board otherDecks do: [:each | each attack: self].! !

!Witch methodsFor: 'game' stamp: 'JohnParker 7/19/2012 20:33'!
resolveAttack: args board: board deck: deck
	(playersResolved at: deck number) ifTrue: [self error: 'That attack has already been resolved.'].
	deck gainCard: Curse to: #discard.
	
	playersResolved at: deck number put: true.
	board currentDeck cardResolved: (playersResolved inject: true into: [:val :each | val and: each]).
	deck attack: nil.
	! !

ActionCard subclass: #Cellar
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Cellar class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 22:04'!
cardName
	^'Cellar'! !

!Cellar class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 23:30'!
cost
	^2! !

!Cellar methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:17'!
playCard: board
	board currentDeck addActions: 1.
	board currentDeck cardResolved: false.
	board currentDeck triggerEvent: #resolveAction: withArguments: {self class}.
	! !

!Cellar methodsFor: 'game' stamp: 'JohnParker 7/20/2012 15:42'!
resolve: args board: board
	| deck discards|
	deck := board currentDeck.
	
	discards := args at: #discards.
	
	(deck handContainsAll: discards) ifFalse: [
		self error: 'You can only discard cards that you have in your hand.'
	].
	
	discards do: [:each | deck moveCard: each from: #hand to: #discard].
	
	1 to: discards size do: [:a | deck drawCardTo: #hand].
	board currentDeck cardResolved: true.! !

ActionCard subclass: #Chancellor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Chancellor class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:06'!
cardName
	^'Chancellor'! !

!Chancellor class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:06'!
cost
	^3! !

!Chancellor methodsFor: 'game' stamp: 'JohnParker 7/20/2012 12:06'!
playCard: board
	board currentDeck addMoney: 2.
	board currentDeck cardResolved: false.
	board currentDeck triggerEvent: #resolveAction: withArguments: {self class}.
	! !

!Chancellor methodsFor: 'game' stamp: 'JohnParker 7/20/2012 12:09'!
resolve: args board: board
	| deck doDiscard|
	deck := board currentDeck.
	
	doDiscard := args at: #doDiscard.
	
	doDiscard ifTrue: [
		deck drawSize timesRepeat: [deck drawCardTo: #discard].
	]. 
	
	board currentDeck cardResolved: true. ! !

ActionCard subclass: #Chapel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Chapel class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 11:47'!
cardName
	^'Chapel'! !

!Chapel class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 11:46'!
cost
	^2! !

!Chapel methodsFor: 'game' stamp: 'JohnParker 7/20/2012 11:47'!
playCard: board
	board currentDeck cardResolved: false.
	board currentDeck triggerEvent: #resolveAction: withArguments: {self class}.
	! !

!Chapel methodsFor: 'game' stamp: 'JohnParker 7/20/2012 15:43'!
resolve: args board: board
	| deck trashCards|
	deck := board currentDeck.
	
	trashCards := args at: #trash.
	
	(trashCards size <= 4) ifFalse: [
		self error: 'The Chapel can only trash up to 4 cards.'
	].
	
	(deck handContainsAll: trashCards) ifFalse: [
		self error: 'You can only trash cards that you have in your hand.'
	].
	
	trashCards do: [:each | deck trashCard: each from: #hand].
	
	board currentDeck cardResolved: true. ! !

ActionCard subclass: #CouncilRoom
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!CouncilRoom class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 16:28'!
cardName
	^'CouncilRoom'! !

!CouncilRoom class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 16:28'!
cost
	^5! !

!CouncilRoom methodsFor: 'game' stamp: 'JohnParker 7/20/2012 16:30'!
playCard: board
	board currentDeck addBuys: 1.
	4 timesRepeat: [board currentDeck drawCardTo: #hand].
	board otherDecks do: [:each | each drawCardTo: #hand].! !

ActionCard subclass: #Feast
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Feast class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:22'!
cardName
	^'Feast'! !

!Feast class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:22'!
cost
	^4! !

!Feast methodsFor: 'game' stamp: 'JohnParker 7/20/2012 12:22'!
playCard: board
	board currentDeck cardResolved: false.
	board currentDeck triggerEvent: #resolveAction: withArguments: {self class}.
	! !

!Feast methodsFor: 'game' stamp: 'JohnParker 7/20/2012 15:44'!
resolve: args board: board
	| deck gainCard|
	deck := board currentDeck.
	
	gainCard := args at: #gainCard.

	(gainCard cost > 5) ifTrue: [
		self error: 'You can only gain a card costing up to 5.'.
	].
	
	deck trashCard: self class from: #play.
	deck gainCard: gainCard to: #discard.
	
	board currentDeck cardResolved: true.! !

ActionCard subclass: #Festival
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Festival class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 16:35'!
cardName
	^'Fesitval'! !

!Festival class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 16:35'!
cost
	^5! !

!Festival methodsFor: 'game' stamp: 'JohnParker 7/20/2012 19:25'!
playCard: board
	((board currentDeck addActions: 2) addBuys: 1) addMoney: 2.! !

ActionCard subclass: #Laboratory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Laboratory class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 16:32'!
cardName
	^'Laboratory'! !

!Laboratory class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 16:32'!
cost
	^5! !

!Laboratory methodsFor: 'game' stamp: 'JohnParker 7/20/2012 16:32'!
playCard: board
	board currentDeck addActions: 1.
	2 timesRepeat: [board currentDeck drawCardTo: #hand].! !

ActionCard subclass: #Library
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Library class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 19:46'!
cardName
	^'Library'! !

!Library class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 19:46'!
cost
	^5! !

!Library methodsFor: 'game' stamp: 'JohnParker 7/20/2012 19:47'!
playCard: board
	board currentDeck cardResolved: false.
	self resolve: nil board: board.
	! !

!Library methodsFor: 'game' stamp: 'JohnParker 7/20/2012 19:53'!
resolve: args board: board
	| deck discard lastDrawn|
	deck := board currentDeck.
	
	lastDrawn := nil.
	[(deck hand size < 7) and: (lastDrawn class inheritsFrom: ActionCard) not] whileTrue: [
		deck drawCardTo: #reveal.
		lastDrawn := deck reveal last.
		(lastDrawn class inheritsFrom: ActionCard) ifFalse: [deck moveCard: lastDrawn class from: #reveal to: #hand].
	].	
	self error: 'Not done implementing!!!!!!'.
	
	board currentDeck cardResolved: true.! !

ActionCard subclass: #Mine
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Mine class methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/17/2012 21:50'!
cardName
	^'Mine'! !

!Mine class methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 12:48'!
cost
	^5! !

!Mine methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:18'!
playCard: board
	board currentDeck cardResolved: false.
	board currentDeck triggerEvent: #resolveAction: withArguments: {self class}.
	! !

!Mine methodsFor: 'game' stamp: 'JohnParker 7/20/2012 15:46'!
resolve: args board: board
	| deck trashCard gainCard|
	deck := board currentDeck.
	
	trashCard := args at: #trashCard.
	gainCard := args at: #gainCard.
	
	(deck handContains: trashCard) ifFalse: [
		self error: 'You do not have a ' , trashCard cardName , 'in your hand to trash.'.
		^self.
	].
	
	((trashCard inheritsFrom: TreasureCard) and: (gainCard inheritsFrom: TreasureCard)) ifFalse: [
		self error: 'Mine only works on treasure cards.'.
		^self.
	].

	((trashCard cost + 3) >= gainCard cost) ifFalse: [
		self notifyBadResolve: args board: board message: ('You can only gain a treasure costing at most 3 more than the trashed card.').
		^self.
	].
	
	deck trashCard: trashCard from: #hand.
	deck gainCard: gainCard to: #hand.
	
	board currentDeck cardResolved: true.! !

ActionCard subclass: #Moneylender
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Moneylender class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 13:06'!
cardName
	^'Moneylender'! !

!Moneylender class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 13:06'!
cost
	^4! !

!Moneylender methodsFor: 'game' stamp: 'JohnParker 7/20/2012 13:08'!
playCard: board
	|deck|
	deck := board currentDeck.
	(deck trashCard: Copper from: #hand) ifTrue: [
		deck addMoney: 3.
	].
	
	! !

ActionCard subclass: #Remodel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Remodel class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 16:14'!
cardName
	^'Remodel'! !

!Remodel class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 16:14'!
cost
	^4! !

!Remodel methodsFor: 'game' stamp: 'JohnParker 7/20/2012 16:13'!
playCard: board
	board currentDeck cardResolved: false.
	board currentDeck triggerEvent: #resolveAction: withArguments: {self class}.
	! !

!Remodel methodsFor: 'game' stamp: 'JohnParker 7/20/2012 16:19'!
resolve: args board: board
	| deck trashCard gainCard|
	deck := board currentDeck.
	
	trashCard := args at: #trashCard.
	gainCard := args at: #gainCard.
	
	(deck handContains: trashCard) ifFalse: [
		self error: 'You do not have a ' , trashCard cardName , 'in your hand to trash.'.
		^self.
	].

	((trashCard cost + 2) >= gainCard cost) ifFalse: [
		self notifyBadResolve: args board: board message: ('You can only gain a card costing at most 2 more than the trashed card.').
		^self.
	].
	
	deck trashCard: trashCard from: #hand.
	deck gainCard: gainCard to: #discard.
	
	board currentDeck cardResolved: true.! !

ActionCard subclass: #ResponseCard
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

ResponseCard subclass: #Moat
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Moat class methodsFor: 'accessing' stamp: 'JohnParker 7/18/2012 20:01'!
cardName
	^'Moat'! !

!Moat class methodsFor: 'accessing' stamp: 'JohnParker 7/18/2012 20:01'!
cost
	^2! !

!Moat methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:19'!
playCard: board
	|deck|
	deck := board currentDeck.
	2 timesRepeat: [deck drawCardTo: #hand].
	! !

!Moat methodsFor: 'game' stamp: 'JohnParker 7/18/2012 20:00'!
revealResponse: board deck: deck
	deck attack: nil.! !

!ResponseCard methodsFor: 'game' stamp: 'JohnParker 7/18/2012 19:57'!
revealResponse: board deck: deck! !

ActionCard subclass: #Smithy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Smithy class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 13:57'!
cardName
	^'Smithy'! !

!Smithy class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 23:30'!
cost
	^4! !

!Smithy methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:19'!
playCard: board
	3 timesRepeat: [board currentDeck drawCardTo: #hand].! !

ActionCard subclass: #ThroneRoom
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!ThroneRoom class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 19:28'!
cardName
	^'Throne Room'! !

!ThroneRoom class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 19:28'!
cost
	^4! !

!ThroneRoom methodsFor: 'game' stamp: 'JohnParker 7/20/2012 19:28'!
playCard: board
	board currentDeck cardResolved: false.
	board currentDeck triggerEvent: #resolveAction: withArguments: {self class}.
	! !

!ThroneRoom methodsFor: 'game' stamp: 'JohnParker 7/20/2012 19:35'!
resolve: args board: board
	| deck playCard card|
	deck := board currentDeck.
	
	playCard := args at: #playCard.
	
	(deck handContains: playCard) ifFalse: [
		self error: 'You do not have a ' , playCard cardName , 'in your hand.'.
	].
	
	(playCard inheritsFrom: ActionCard) ifFalse: [
		self error: 'Throne Room can only be used to play Action cards twice.'.
	].

	card := deck hand detect: [:each | each class = playCard].
	card playCard: board.
	card play: board.
	
	board currentDeck cardResolved: true.! !

ActionCard subclass: #Village
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Village class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 00:07'!
cardName
	^'Village'! !

!Village class methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 23:30'!
cost
	^3! !

!Village methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:19'!
playCard: board
	|deck|
	deck := board currentDeck.
	deck drawCardTo: #hand.
	deck addActions: 2.
	! !

ActionCard subclass: #Woodcutter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Woodcutter class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:13'!
cardName
	^'Woodcutter'! !

!Woodcutter class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:13'!
cost
	^3! !

!Woodcutter methodsFor: 'game' stamp: 'JohnParker 7/20/2012 12:14'!
playCard: board
	|deck|
	deck := board currentDeck.
	deck addMoney: 2.
	deck addBuys: 1.
	! !

ActionCard subclass: #Workshop
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Workshop class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:22'!
cardName
	^'Workshop'! !

!Workshop class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:22'!
cost
	^3! !

!Workshop methodsFor: 'game' stamp: 'JohnParker 7/20/2012 12:16'!
playCard: board
	board currentDeck cardResolved: false.
	board currentDeck triggerEvent: #resolveAction: withArguments: {self class}.
	! !

!Workshop methodsFor: 'game' stamp: 'JohnParker 7/20/2012 15:45'!
resolve: args board: board
	| deck gainCard|
	deck := board currentDeck.
	
	gainCard := args at: #gainCard.

	(gainCard cost > 4) ifTrue: [
		self error: 'You can only gain a card costing up to 4.'.
		^self.
	].
	
	deck gainCard: gainCard to: #discard.
	
	board currentDeck cardResolved: true.! !

!Card class methodsFor: 'game' stamp: 'JohnParker 7/18/2012 12:08'!
buy: board
	^board currentDeck gainCard: self to: #discard.! !

!Card class methodsFor: 'accessing' stamp: 'JohnParker 7/3/2012 20:37'!
cardName
	^'Card' ! !

!Card class methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/14/2012 08:27'!
color
	^self subclassResponsibility! !

!Card class methodsFor: 'accessing' stamp: 'JohnParker 7/3/2012 20:37'!
cost
	^0.! !

!Card class methodsFor: 'instance creation' stamp: 'JohnParker 7/3/2012 20:03'!
new
	self error: 'Cards can only be created by the Board'! !

!Card methodsFor: 'accessing' stamp: 'JohnParker 7/3/2012 20:37'!
cardName
	^self class cardName.! !

!Card methodsFor: 'accessing' stamp: 'JohnParker 7/3/2012 20:37'!
cost
	^self class cost.! !

!Card methodsFor: 'game' stamp: 'JohnParker 7/18/2012 11:47'!
play: board
	|ret|
	(board currentDeck moveCard:  self class from: #hand to: #play) ifFalse: [self Error: 'You do not have a ' , self cardName , ' in your hand!!'].
	
	[ret := self playCard: board] on: Error do: [:e | 
		board currentDeck moveCard: self class from: #play to: #hand.
		e pass.
	].
	^ret.! !

!Card methodsFor: 'private' stamp: 'JohnParker 7/18/2012 11:47'!
playCard: board
	self SubclassResponsibility ! !

Card subclass: #TreasureCard
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!TreasureCard commentStamp: 'JohnParker 7/4/2012 15:05' prior: 0!
A TreasureCard is an abstract class representing a Dominion treasure card.  It adds the value message which returns the amount of treasure the card is worth.!

TreasureCard subclass: #Copper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Copper class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
cardName 
	^'Copper'! !

!Copper class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:45'!
cost
	^0! !

!Copper methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:22'!
value
	^1! !

TreasureCard subclass: #Gold
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Gold class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
cardName 
	^'Gold'! !

!Gold class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
cost
	^6! !

!Gold methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
value
	^3! !

TreasureCard subclass: #Silver
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Silver class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
cardName 
	^'Silver'! !

!Silver class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:59'!
cost
	^3! !

!Silver methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:58'!
value
	^2! !

!TreasureCard class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:40'!
cardName
	^'Treasure'! !

!TreasureCard class methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/14/2012 08:28'!
color
	^Color yellow! !

!TreasureCard methodsFor: 'private' stamp: 'JohnParker 7/18/2012 11:58'!
playCard: board
	board currentDeck addMoney: self value.! !

!TreasureCard methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:07'!
value
	self SubclassResponsibility! !

Card subclass: #VictoryCard
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!VictoryCard commentStamp: 'JohnParker 7/4/2012 15:06' prior: 0!
A VictoryCard is an abstract class representing a Dominion victory card.  It adds the value message which returns the number of victory points the card is worth. 
It also overrides the playCard message to generate an error since victory cards can not be played.!

VictoryCard subclass: #Curse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Curse class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:06'!
cardName 
	^'Curse'! !

!Curse class methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/17/2012 22:40'!
color
	^Color magenta! !

!Curse class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:06'!
cost
	^0! !

!Curse methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:29'!
value: board deck: deck
	^-1! !

VictoryCard subclass: #Duchy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Duchy class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:00'!
cardName 
	^'Duchy'! !

!Duchy class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:00'!
cost
	^5! !

!Duchy methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:29'!
value: board deck: deck
	^3! !

VictoryCard subclass: #Estate
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Estate class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:49'!
cardName 
	^'Estate'! !

!Estate class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:49'!
cost
	^2! !

!Estate methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:30'!
value: board deck: deck
	^1! !

VictoryCard subclass: #Gardens
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Gardens class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:28'!
cardName 
	^'Gardens'! !

!Gardens class methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:28'!
cost
	^4! !

!Gardens methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:32'!
value: board deck: deck
	^(deck allCards size / 10) floor.! !

VictoryCard subclass: #Province
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Cards'!

!Province class methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/15/2012 17:18'!
cardName 
	^'Province'! !

!Province class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 16:00'!
cost
	^8! !

!Province methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:30'!
value: board deck: deck
	^6! !

!VictoryCard class methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:47'!
cardName 
	^'Victory'! !

!VictoryCard class methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/14/2012 08:28'!
color
	^Color green! !

!VictoryCard methodsFor: 'private' stamp: 'JohnParker 7/18/2012 11:58'!
playCard: board
	self Error: 'Victory cards can not be played'! !

!VictoryCard methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 15:07'!
value
	self SubclassResponsibility ! !

Object subclass: #Command
	instanceVariableNames: 'deckNumber'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!Command commentStamp: 'JohnParker 7/4/2012 14:53' prior: 0!
A Command is an abstract class implementing the command object pattern.  Its reciever is specified by deckNumber.

Instance Variables
	deckNumber:		Number

deckNumber
	- The deck number of the reciever.
!

Command subclass: #CardCommand
	instanceVariableNames: 'cardClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!CardCommand commentStamp: 'JohnParker 7/4/2012 14:52' prior: 0!
A CardCommand is an abstract class representing a Command that accepts a card class as a parameter

Instance Variables
	cardClass:		Card class

cardClass
	- passed to the reciever as an argument
!

CardCommand subclass: #ActionCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!ActionCommand commentStamp: 'JohnParker 7/4/2012 14:50' prior: 0!
An ActionCommand is a command object that sends the playAction: cardClass message to the deck specified by deckNumber!

!ActionCommand methodsFor: 'run' stamp: 'JohnParker 7/20/2012 12:59'!
do: board
	[(board deck: deckNumber) playAction: cardClass] on: Error do: [:e | 
		(board deck: deckNumber) triggerEvent: #command:error: withArguments: {self. e}.
	].
	! !

CardCommand subclass: #BuyCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!BuyCommand commentStamp: 'JohnParker 7/4/2012 14:51' prior: 0!
A BuyCommand is a command object that sends the buy: cardClass message to the deck specified by deckNumber!

!BuyCommand methodsFor: 'run' stamp: 'JohnParker 7/18/2012 12:02'!
do: board
	[(board deck: deckNumber) buy: cardClass] on: Error do: [:e | 
		(board deck: deckNumber) triggerEvent: #command:error: withArguments: {self. e messageText}.
	].
	! !

!CardCommand class methodsFor: 'instance creation' stamp: 'JohnParker 7/4/2012 14:20'!
deck: aNumber card: aCardClass
	^self new deck: aNumber card: aCardClass.! !

!CardCommand methodsFor: 'initialize-release' stamp: 'JohnParker 7/4/2012 14:20'!
deck: aNumber card: aCardClass
	deckNumber := aNumber.
	cardClass := aCardClass.
	! !

CardCommand subclass: #RevealResponseCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!RevealResponseCommand methodsFor: 'run' stamp: 'JohnParker 7/20/2012 15:40'!
do: board
	[(board deck: deckNumber) revealResponse: cardClass] on: Error do: [:e | 
		(board deck: deckNumber) triggerEvent: #command:error: withArguments: {self. e}.
	].! !

CardCommand subclass: #TreasureCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!TreasureCommand commentStamp: 'JohnParker 7/4/2012 14:51' prior: 0!
A TreasureCommand is a command object that sends the playTreasure: cardClass message to the deck specified by deckNumber!

!TreasureCommand methodsFor: 'run' stamp: 'JohnParker 7/20/2012 15:40'!
do: board
	[(board deck: deckNumber) playTreasure: cardClass] on: Error do: [:e | 
		(board deck: deckNumber) triggerEvent: #command:error: withArguments: {self. e}.
	].
	
	! !

!Command class methodsFor: 'instance creation' stamp: 'JohnParker 7/4/2012 14:20'!
deck: aNumber
	^self new deck: aNumber.! !

!Command methodsFor: 'initialize-release' stamp: 'JohnParker 7/4/2012 14:21'!
deck: aNumber
	deckNumber := aNumber.
	! !

!Command methodsFor: 'run' stamp: 'JohnParker 7/18/2012 12:01'!
do: board
	self SubclassResponsibility.! !

Command subclass: #EndTurnCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!EndTurnCommand commentStamp: 'JohnParker 7/4/2012 15:04' prior: 0!
An EndTurnCommand is a command object that sends the endTurn message to the deck specified by deckNumber
!

!EndTurnCommand methodsFor: 'run' stamp: 'JohnParker 7/18/2012 12:03'!
do: board
	[(board deck: deckNumber) cleanup] on: Error do: [:e | 
		(board deck: deckNumber) triggerEvent: #command:error: withArguments: {self. e messageText}.
	].! !

Command subclass: #ResolveActionCommand
	instanceVariableNames: 'args'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!ResolveActionCommand commentStamp: 'JohnParker 7/10/2012 19:57' prior: 0!
A ResolveActionCommand is a command sent by a player to inform their deck of the way in which they would like to resolve the action in play.  Should be sent in response to a #resolveAction: event from their Deck.

Instance Variables
	argVector:		Array

argVector
	- The list of arguments to be passed.  May vary from card to card.
!

!ResolveActionCommand class methodsFor: 'instance creation' stamp: 'JohnParker 7/6/2012 00:19'!
deck: aNumber argVector: anArgVector
	^self new deck: aNumber argVector: anArgVector! !

!ResolveActionCommand class methodsFor: 'instance creation' stamp: 'JohnParker 7/18/2012 21:32'!
deck: aNumber args: aDict
	^self new deck: aNumber args: aDict! !

!ResolveActionCommand methodsFor: 'initialize-release' stamp: 'JohnParker 7/18/2012 21:32'!
deck: aNumber args: aDict
	deckNumber := aNumber.
	args := aDict.! !

!ResolveActionCommand methodsFor: 'run' stamp: 'JohnParker 7/20/2012 15:40'!
do: board
	[(board deck: deckNumber) resolveAction: args] on: Error do: [:e | 
		self halt.
		(board deck: deckNumber) triggerEvent: #command:error: withArguments: {self. e}.
	].
	! !

Command subclass: #ResolveAttackCommand
	instanceVariableNames: 'args'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!ResolveAttackCommand class methodsFor: 'instance creation' stamp: 'JohnParker 7/18/2012 12:53'!
deck: aNumber args: aDict
	^self new deck: aNumber args: aDict! !

!ResolveAttackCommand methodsFor: 'initialize-release' stamp: 'JohnParker 7/18/2012 12:54'!
deck: aNumber args: aDict
	deckNumber := aNumber.
	args := aDict.! !

!ResolveAttackCommand methodsFor: 'run' stamp: 'JohnParker 7/20/2012 15:40'!
do: board
	[(board deck: deckNumber) resolveAttack: args] on: Error do: [:e | 
		self halt.
		(board deck: deckNumber) triggerEvent: #command:error: withArguments: {self. e}.
	].
	! !

Object subclass: #Deck
	instanceVariableNames: 'deckNumber board areas actions buys phase money resolved attackedBy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion'!

!Deck commentStamp: 'JohnParker 7/17/2012 22:01' prior: 0!
A Deck represents all the cards in a Dominion Player's possesion and that player's current turn state.  A card can either be in the draw, discard, hand, play, peek, or reveal areas.  It can only be in one.  A card can only be removed from the deck by being trashed, which is to say it is moved to the trash area of the board.

Instance Variables
	areas:		Dictionary: Symbol -> CardArea
	actions: 	Number
	buys: 		Number
	money: 	Number
	phase: 		Symbol
	resolving:	Boolean
	
areas
	- This is a dictionary containing all of the places a card in a players posession may be.  Only the following symbols are allowed as keys:		
	#discard
		- The players discard pile.

	#draw
		- The players draw pile.  When this pile is empty the discard pile is shuffled and becomes the new draw pile.

	#hand
		- The player's current hand.  Drawn as a set of 5 cards during the cleanup phase.
	
	#play
		- The play area.  Played cards are placed in this area until the cleanup phase.
	
	#peek
		- A list of cards currently being "peeked" at by the player.  Used by cards that require a player to look at cards from their deck and make a decision.
	
	#reveal
		- A list of cards currently being "revealed" by the player.  Used by cards that require a player to reveal cards from their deck.


actions
	- The number of actions remaining to be played.

buys
	- The number of buys a player can still make.

money
	- The amount of money the player has this turn due to playing treasure or action cards that give extra money.

phase
	- The phase of the current Turn.  Can be one of:
		#Action 	The player can play actions, treasure, buy cards, or end their turn during this phase
		#Treasure 	The player can play treasure, buy cards, or end their turn during this phase
		#Buy 		The player can buy cards or end their turn during this phase
		#None 		It is not currently the players turn.  Any action will generate an error.

resolving
	- True if the deck is waiting for the player to resolve the last card played.

!

!Deck class methodsFor: 'instance creation' stamp: 'JohnParker 7/18/2012 11:43'!
number: aNumber board: aBoard
	^self new number: aNumber board: aBoard.! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 00:11'!
actions
	^actions! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 00:06'!
addActions: aNumber
	actions := actions + aNumber.! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 00:07'!
addBuys: aNumber
	buys := buys + aNumber! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 01:43'!
addMoney: aNumber
	money := money + aNumber! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:48'!
allCards
	|all|
	all := OrderedCollection new.
	areas do: [:each | all addAll: each].
	^all.! !

!Deck methodsFor: 'attacks' stamp: 'JohnParker 7/19/2012 20:32'!
attack: aCard
	attackedBy := aCard.
	aCard isNil ifFalse: [self triggerEvent: #attack: withArguments: {aCard class}. ]! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/22/2012 21:00'!
buy: aCardClass
	(board currentDeck = self) ifFalse: [self error: 'It is not your turn!!'].
	(aCardClass cost > money) ifTrue: [self error: 'You do not have enough money to purchase a ' , aCardClass cardName].
	(buys = 0) ifTrue: [self cleanup. self error: 'You are out of buys.'].
	resolved ifFalse: [self error: 'You can not buy a card while resolving another card.'].
	
	(aCardClass buy: board) ifTrue: [
		buys := buys - 1.
		money := money - aCardClass cost.
		(buys = 0) ifTrue: [self cleanup].
	] ifFalse: [self error: 'Could not buy a ' , aCardClass cardName]
	
	
	! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 00:11'!
buys
	^buys! !

!Deck methodsFor: 'private' stamp: 'JohnParker 7/17/2012 20:54'!
card: aCardClass from: area1 to: area2
	|card|
	card := (areas at: area1) detect: [:each | each class = aCardClass] ifNone: [^false].
	(areas at: area1) remove: card ifAbsent: [^false].
	(areas at: area2) addLast: card.
	^true.! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:11'!
cardResolved: val
	resolved := val! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:11'!
cleanup
	(board currentDeck = self) ifFalse: [self error: 'It is not your turn!!'].
	resolved ifFalse: [self error: 'You can not end your turn while resolving a card.'].
	self default.
	board nextDeck.
	! !

!Deck methodsFor: 'private' stamp: 'JohnParker 7/18/2012 21:14'!
default
	actions := 1.
	buys := 1.
	money := 0.
	phase := #None.
	resolved := true.
	
	(areas at: #discard) addAllLast: self play.
	self play removeAll.
	
	(areas at: #discard) addAllLast: self hand.
	self hand removeAll.
	
	1 to: 5 do: [:a | self drawCardTo: #hand].! !

!Deck methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/20/2012 09:45'!
discardCount
	^ (areas at: #discard) size! !

!Deck methodsFor: 'private' stamp: 'JohnParker 7/17/2012 20:56'!
drawCard
	| card draw discard|
	"Reshuffle the discard into the draw if the draw is empty"
	draw := areas at: #draw.
	discard := areas at: #discard.
	
	(draw size = 0) ifTrue: [
		draw addAllLast: discard. 
		discard removeAll. 
		draw shuffle.
		self triggerEvent: #deckShuffled: withArguments: {deckNumber}.
	].
	"If the draw is still empty, we have no cards to draw so do nothing"
	(draw size = 0) ifFalse: [
		card := draw last.
		draw removeLast.
	].
	self  triggerEvent: #cardDrawn: withArguments: {card class}.
	^card.
	! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/17/2012 21:35'!
drawCardTo: anArea
	| card |
	card := self drawCard.
	card isNil ifFalse: [
		(areas at: anArea) add: card.
	].

	self triggerMoveEvent: card class from: #draw to: anArea.
	^card.
	! !

!Deck methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/20/2012 09:45'!
drawCount
	^ (areas at: #draw) size! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/20/2012 12:13'!
drawSize
	^(areas at: #draw) size.! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/18/2012 11:45'!
gainCard: aCardClass to: anArea
	|card|
	card := board takeCard: aCardClass deck: deckNumber.
	card isNil ifFalse: [
		(areas at: anArea) addLast: card.
		self triggerEvent: #deck:cardMovedGlobal:from:to: withArguments: {deckNumber. aCardClass. #board. anArea}.
		^true.
	].
	^false.
! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:48'!
hand
	^areas at: #hand! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:48'!
handContains: aCardClass
	^(self hand detect: [:each | each class = aCardClass] ifNone: [nil]) isNil not.
	! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:49'!
handContainsAll: aCardClassVector
	|set|
	set := aCardClassVector asSet.
	^set = (set select: [:testCardClass | 
		(self hand count: [:handCard | handCard class = testCardClass]) >=  
			(aCardClassVector count: [:each | each class = testCardClass])
	]).
	! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 13:22'!
money
	^money! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/17/2012 21:57'!
moveCard: aCardClass from: area1 to: area2
	|card|
	
	((areas at: area1) visibility = #hidden) ifTrue: [^false].
	card := (areas at: area1) detect: [:each | each class = aCardClass] ifNone: [^false].
	(areas at: area1) remove: card ifAbsent: [^false].
	(areas at: area2) addLast: card.
	
	self triggerMoveEvent: aCardClass from: area1 to: area2.
	^true.! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/5/2012 20:29'!
number
	^deckNumber! !

!Deck methodsFor: 'initialize-release' stamp: 'JohnParker 7/18/2012 11:42'!
number: aNumber board: aBoard
	deckNumber := aNumber.
	board := aBoard.
	areas := Dictionary new.
	areas at: #discard put: (CardArea visibility: #global).
	areas at: #draw put: (CardArea visibility: #hidden).
	areas at: #hand put: (CardArea visibility: #player).
	areas at: #play put: (CardArea visibility: #global).
	areas at: #peek put: (CardArea visibility: #player).
	areas at: #reveal put: (CardArea visibility: #global).
	
	1 to: 7 do: [:a | self gainCard: Copper to: #discard].
	1 to: 3 do: [:a | self gainCard: Estate to: #discard].
	self default.
	
	! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/4/2012 12:45'!
phase
	^phase! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:49'!
play
	^areas at: #play! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:20'!
playAction: aCardClass
	| card |
	(board currentDeck = self) ifFalse: [self error: 'It is not your turn!!'].
	(aCardClass inheritsFrom: ActionCard) ifFalse: [self error: aCardClass cardName , ' is not an action card.'].
	(phase = #Treasure) ifTrue: [self error: 'You can not play an action during the treasure phase.'].
	(phase = #Buy) ifTrue: [self error: 'You can not play an action during the buy phase.'].
	resolved ifFalse: [self error: 'You can not play an action while resolving another card.'].
	
	card := self hand detect: [:each | each class = aCardClass] ifNone: [self error: 'You do not have a ' , aCardClass cardName , ' in your hand.'].
	
	actions := actions - 1.
	
	card play: board.
	
	(actions = 0) ifTrue: [phase := #Treasure].! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:12'!
playTreasure: aCardClass
	| card |
	(board currentDeck = self) ifFalse: [self error: 'It is not your turn!!'].
	(aCardClass inheritsFrom: TreasureCard) ifFalse: [self error: aCardClass cardName , ' is not a treasure card.'].
	(phase = #Buy) ifTrue: [self error: 'You can not play a treasure during the buy phase.'].
	resolved ifFalse: [self error: 'You can not play a treasure while resolving another card.'].
	
	card := self hand detect: [:each | each class = aCardClass] ifNone: [self error: 'You do not have a ' , aCardClass cardName , ' in your hand.'].
	
	card play: board.
	phase := #Treasure.
	
	self triggerEvent: #cardPlayed withArguments: {deckNumber. aCardClass}.! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/18/2012 21:20'!
resolveAction: argVector
	(board currentDeck = self) ifFalse: [self error: 'It is not your turn!!'].
	resolved ifTrue: [self error: 'There is no action to resolve right now.'].
	
	self play last resolve: argVector board: board.
	! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/19/2012 20:34'!
resolveAttack: args
	(board currentDeck = self) ifTrue: [self error: 'You can not be attacked on your turn.'].
	attackedBy ifNil: [^self].
	
	attackedBy resolveAttack: args board: board deck: self.
	! !

!Deck methodsFor: 'accessing' stamp: 'JohnParker 7/19/2012 20:08'!
reveal
	^areas at: #reveal! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/18/2012 20:16'!
revealResponse: aCardClass
	| card |
	(board currentDeck = self) ifTrue: [self error: 'You can not be attacked on your turn.'].
	attackedBy ifNil: [^self].
	
	card := self hand detect: [:each | each class = aCardClass] ifNone: [self error: 'You do not have a ' , aCardClass cardName , ' in your hand.'].
	
	self moveCard: card class from: #hand to: #reveal.
	self moveCard: card class from: #reveal to: #hand.
	card revealResponse: board deck: self.
	
	
	! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/13/2012 13:25'!
startTurn
	phase := #Action.
	self triggerEvent: #playerTurn withArguments: {deckNumber} .! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/22/2012 21:34'!
takeTrashTopTo: anArea
	|card|
	card := board takeTrashTop.
	
	card isNil ifFalse: [
		(areas at: anArea) addLast: card.
		self triggerEvent: #deck:cardMovedGlobal:from:to: withArguments: {deckNumber. card class. #trash. anArea}.
		^true.
	].
	^false.
! !

!Deck methodsFor: 'card-moving' stamp: 'JohnParker 7/18/2012 11:45'!
trashCard: aCardClass from: anArea
	|card|
	card := (areas at: anArea) detect: [:each | each class = aCardClass] ifNone: [nil].
	card isNil ifFalse: [
		(areas at: anArea) remove: card.
		board trashCard: card deck: aCardClass.
		self triggerEvent: #deck:cardMovedGlobal:from:to: withArguments: {deckNumber. aCardClass. anArea. #trash}.
		^true.
	].
	^false.
! !

!Deck methodsFor: 'private' stamp: 'JohnParker 7/17/2012 22:10'!
triggerMoveEvent: aCardClass from: area1 to: area2
	(((areas at: area1) visibility = #global) or: ((areas at: area2) visibility = #global)) ifTrue: [
		self triggerEvent: #deck:cardMovedGlobal:from:to: withArguments: {deckNumber. aCardClass. area1. area2}.
	] ifFalse: [
		self triggerEvent: #deck:cardMovedPlayer:from:to: withArguments: {deckNumber. aCardClass. area1. area2}.
	].! !

!Deck methodsFor: 'game' stamp: 'JohnParker 7/20/2012 12:30'!
victoryPoints
	(board gameOver) ifFalse: [self error: 'No peeking!! The game is not over yet!!'].
	^self allCards inject: 0 into: [:points :card |
		(card class inheritsFrom: VictoryCard) ifTrue: [points + (card value: board deck: self)] ifFalse: [points]
	]! !

!Deck methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:58'!
z_addCardToHand: aCardClass
	self hand add: aCardClass basicNew! !

!Deck methodsFor: 'testing' stamp: 'JohnParker 7/20/2012 13:13'!
z_areas
	^areas! !

!Deck methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:59'!
z_discard
	^(areas at: #discard)! !

!Deck methodsFor: 'testing' stamp: 'JohnParker 7/17/2012 20:59'!
z_draw
	^(areas at: #draw)! !

!Deck methodsFor: 'testing' stamp: 'JohnParker 7/18/2012 21:12'!
z_resolving 
	^resolved not ! !

Object subclass: #KingdomRenderer
	instanceVariableNames: 'startingPoint currentPosition remaining rect row kingdom rowWidth rowHeight'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!KingdomRenderer methodsFor: 'computing' stamp: 'MichaelOSullivan 7/20/2012 10:20'!
compute
	| keys|

	startingPoint := kingdom innerBounds origin translateBy: 5 @ 5.
	kingdom title position: startingPoint.
	currentPosition := kingdom title position translateBy: 0 @ (kingdom title height + 20).
	
	keys := OrderedCollection new.
	kingdom cards keysDo: [:each | keys add: each].
	keys asSortedCollection do: [:each | self processRow: (kingdom cards at: each)].
	
	rect := Rectangle 	origin: (kingdom innerBounds origin)
				    		corner: (currentPosition translateBy: rowWidth @ 0).
	kingdom bounds: rect! !

!KingdomRenderer methodsFor: 'computing' stamp: 'MichaelOSullivan 7/17/2012 23:00'!
doCarriageFeedLineReturn
	"moves the current position down a row and back to the far left of the area"

	currentPosition := currentPosition translateBy: (startingPoint x - currentPosition x) @ (rowHeight + 20)! !

!KingdomRenderer methodsFor: 'initialize' stamp: 'MichaelOSullivan 7/18/2012 22:29'!
initializeWith: aKingdom
	kingdom := aKingdom.
	rowWidth := 0.
	rowHeight := 0! !

!KingdomRenderer methodsFor: 'computing' stamp: 'MichaelOSullivan 7/18/2012 22:48'!
processRow: aCollection
	aCollection do: [:each |  self renderCard: each].
	rowWidth := rowWidth max: currentPosition x.
	self doCarriageFeedLineReturn! !

!KingdomRenderer methodsFor: 'computing' stamp: 'MichaelOSullivan 7/17/2012 23:21'!
renderCard: aCard

	aCard position: currentPosition.
	currentPosition  := currentPosition translateBy: (aCard width + 20) @ 0.
	rowHeight := rowHeight max: aCard height! !

Object subclass: #TestPlayer
	instanceVariableNames: 'board deckNumber errorCommand errorString events'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Test'!

!TestPlayer commentStamp: 'JohnParker 7/10/2012 19:56' prior: 0!
A Player is an abstract class that represents a player.  In this design, a player controls the corresponding deck, sending commands to it in order to play and respond to cards.  A Player also listens for events from Board and Deck which inform it of changes to the game state.  The player may also display the current state of the board if it is a human player.

Instance Variables
	deckNumber:		Number

deckNumber
	- The number of the deck this player controls.
!

!TestPlayer methodsFor: 'events' stamp: 'JohnParker 7/22/2012 21:46'!
attack: aCardClass
	|card deck|
	deck := board deck: deckNumber.
	events addLast: (Message selector: #attack: arguments: {aCardClass}).
	
	card := (board deck: deckNumber) hand detect: [:each | each class inheritsFrom: ResponseCard] ifNone: [nil].
	card isNil ifFalse: [board execute: (RevealResponseCommand deck: deckNumber card: card class)].
	
	(aCardClass = Bureaucrat) ifTrue: [
		|command args|
		card := (board deck: deckNumber) hand detect: [:each | each class inheritsFrom: VictoryCard] ifNone: [command := ResolveAttackCommand deck: deckNumber args: Dictionary new. nil].
		
		(card isNil) ifFalse: [
			args := Dictionary new.
			args at: #bureaucratCard put: card class.
			command := ResolveAttackCommand deck: deckNumber args: args.
		].
		
		board execute: command.
	].

	(aCardClass = Militia) ifTrue: [
		|dict|
		
		dict := Dictionary new.
		dict at: #discards put: OrderedCollection new.
		1 to: (deck hand size - 3) do: [:num |
			(dict at: #discards) add: (deck hand at: num) class.
		]. 
		board execute: (ResolveAttackCommand deck: deckNumber args: dict)
	].
	
	((aCardClass = Witch) or: ((aCardClass = Spy) or: (aCardClass = Thief))) ifTrue: [
		board execute: (ResolveAttackCommand deck: deckNumber args: Dictionary new)
	]! !

!TestPlayer methodsFor: 'setup' stamp: 'JohnParker 7/19/2012 19:56'!
board: aBoard deck: aNumber
	board := aBoard.
	deckNumber := aNumber.
	events := OrderedCollection new.! !

!TestPlayer methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:25'!
clearError
	errorCommand := nil.
	errorString := nil.! !

!TestPlayer methodsFor: 'events' stamp: 'JohnParker 7/19/2012 20:00'!
command: aCommand error: aString
	errorCommand := aCommand.
	errorString := aString.
	events addLast: (Message selector: #command:error: arguments: {aCommand. aString}).! !

!TestPlayer methodsFor: 'events' stamp: 'JohnParker 7/19/2012 20:09'!
doesNotUnderstand: aMessage
	(board events includes: aMessage selector) ifFalse: [super doesNotUnderstand: aMessage].
	events addLast: aMessage.
	! !

!TestPlayer methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:24'!
errorCommand
	^errorCommand! !

!TestPlayer methodsFor: 'accessing' stamp: 'JohnParker 7/17/2012 20:24'!
errorString
	^errorString! !

!TestPlayer methodsFor: 'events' stamp: 'JohnParker 7/20/2012 19:43'!
resolveAction: aCardClass
	|dict deck|
	deck := board deck: deckNumber.
	events addLast: (Message selector: #resolveAction: arguments: {aCardClass}).
	
	dict := Dictionary new.
	
	(aCardClass = Cellar) ifTrue: [
		| victoryCards |
		victoryCards := (board deck: deckNumber) hand select: [:each | each class inheritsFrom: VictoryCard] thenCollect: [:each | each class].
		
		dict at: #discards put: victoryCards asArray.
		board execute: (ResolveActionCommand deck: deckNumber args: dict).
	].

	(aCardClass = Chapel) ifTrue: [
		| trashCards |
		trashCards := (board deck: deckNumber) hand select: [:each | (each class = Copper) or: (each class = Estate)] thenCollect: [:each | each class].
		trashCards removeLast: (trashCards size - 4).
		
		dict at: #trash put: trashCards asArray.
		board execute: (ResolveActionCommand deck: deckNumber args: dict).
	].

	(aCardClass = Chancellor) ifTrue: [
		dict at: #doDiscard put: true.
		board execute: (ResolveActionCommand deck: deckNumber args: dict).
	].
	
	((aCardClass = Workshop) or: (aCardClass = Feast)) ifTrue: [
		| treasureCards |
		
		dict at: #gainCard put: Silver.
		board execute: (ResolveActionCommand deck: deckNumber args: dict).
	].
	
	(aCardClass = Mine) ifTrue: [
		| treasureCards trashCard gainCard |
		treasureCards := (board deck: deckNumber) hand select: [:each | each class inheritsFrom: TreasureCard] thenCollect: [:each | each class].
		trashCard := treasureCards detectMin: [:each | each cost].
		gainCard := {Silver. Gold} detect: [:each | each cost = (trashCard cost + 3)].
		
		dict at: #trashCard put: trashCard.
		dict at: #gainCard put: gainCard.
		board execute: (ResolveActionCommand deck: deckNumber args: dict).
	].

	(aCardClass = Remodel) ifTrue: [
		| trashCard gainCard |
		trashCard := Estate.
		gainCard := Silver.
		
		dict at: #trashCard put: trashCard.
		dict at: #gainCard put: gainCard.
		board execute: (ResolveActionCommand deck: deckNumber args: dict).
	].
	
	(aCardClass = ThroneRoom) ifTrue: [
		| playCard |
		playCard := deck hand detect: [:each | each class inheritsFrom: ActionCard].
		
		dict at: #playCard put: playCard class.
		board execute: (ResolveActionCommand deck: deckNumber args: dict).
	].! !

!TestPlayer methodsFor: 'events' stamp: 'JohnParker 7/22/2012 21:48'!
resolveAction: aCardClass deck: aNumber
	|dict|
	
	events addLast: (Message selector: #resolveAction:deck: arguments: {aCardClass. aNumber}).
	
	dict := Dictionary new.
	
	(aCardClass = Spy) ifTrue: [
		|discard|
		
		discard := (board deck: aNumber) reveal last class inheritsFrom: TreasureCard.
		(deckNumber = aNumber) ifTrue: [discard := discard not].
		
		dict at: #deck put: aNumber.
		dict at: #discard put: discard.
		board execute: (ResolveActionCommand deck: deckNumber args: dict).
	].

	(aCardClass = Thief) ifTrue: [
		|trash|
		
		trash := (board deck: aNumber) reveal detect: [:each | each class inheritsFrom: TreasureCard].
		
		dict at: #deck put: aNumber.
		dict at: #trash put: trash class.
		dict at: #take put: true.
		board execute: (ResolveActionCommand deck: deckNumber args: dict).
	].! !

ScrollPane subclass: #TurnLog
	instanceVariableNames: 'log turn deck'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dominion-Display'!

!TurnLog methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/22/2012 21:22'!
command: aCommand error: aString
	| stream |
	stream := String new writeStream.
	stream
		nextPutAll: log contents;
		nextPutAll: 'Command ';
		print: aCommand;
		nextPutAll: ' issued with error message: ';
		nextPutAll: aString;
		crlf.
	log contents: stream contents.! !

!TurnLog methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/22/2012 21:22'!
deck: aNumber cardMovedGlobal: aCardClass from: area1 to: area2
	| stream |
	stream := String new writeStream.
	stream
		nextPutAll: log contents;
		nextPutAll: 'Global Player ';
		print: aNumber;
		nextPutAll: ' moved card ';
		nextPutAll: aCardClass cardName;
		nextPutAll: ' from ';
		print: area1;
		nextPutAll: ' to ';
		print: area2;
		crlf.
	log contents: stream contents.! !

!TurnLog methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/22/2012 21:22'!
deck: aNumber cardMovedPlayer: aCardClass from: area1 to: area2
	| stream |
	stream := String new writeStream.
	stream
		nextPutAll: log contents;
		nextPutAll: 'Player ';
		print: aNumber;
		nextPutAll: ' moved card ';
		nextPutAll: aCardClass cardName;
		nextPutAll: ' from ';
		print: area1;
		nextPutAll: ' to ';
		print: area2;
		crlf.
	log contents: stream contents.! !

!TurnLog methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/22/2012 21:22'!
deck: aNumber took: aCardClass
	| stream |
	stream := String new writeStream.
	stream
		nextPutAll: log contents;
		nextPutAll: 'Player ';
		print: aNumber;
		nextPutAll: ' took card ';
		nextPutAll: aCardClass cardName;
		crlf.
	log contents: stream contents.! !

!TurnLog methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/22/2012 21:22'!
deck: aNumber trashed: aCardClass
	| stream |
	stream := String new writeStream.
	stream
		nextPutAll: log contents;
		nextPutAll: 'Player ';
		print: aNumber;
		nextPutAll: ' trashed card ';
		nextPutAll: aCardClass cardName;
		crlf.
	log contents: stream contents.! !

!TurnLog methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/22/2012 21:22'!
deckShuffled: aNumber
	| stream |
	stream := String new writeStream.
	stream
		nextPutAll: log contents;
		nextPutAll: 'Deck ';
		print: aNumber;
		nextPutAll: ' was shuffled.';
		crlf.
	log contents: stream contents.! !

!TurnLog methodsFor: 'initialize' stamp: 'MichaelOSullivan 7/22/2012 21:49'!
initialize
	super initialize.
	turn := 0.
	deck := 0.
	log := TextMorph new contents: ''.
	log autoFit: true.
	self scroller addMorph: log.! !

!TurnLog methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/22/2012 21:21'!
resolveAction: aCardClass
	| stream |
	stream := String new writeStream.
	stream
		nextPutAll: log content;
		nextPutAll: 'Please resolve card ';
		nextPutAll: aCardClass cardName;
		crlf.
	log contents: stream contents.! !

!TurnLog methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/22/2012 21:22'!
respondToCard: UNDEFINED
	| stream |
	stream := String new writeStream.
	stream
		nextPutAll: log content;
		nextPutAll: 'respondtoCard';
		crlf.
	log contents: stream contents.! !

!TurnLog methodsFor: 'accessing' stamp: 'MichaelOSullivan 7/22/2012 16:07'!
setDeck: aNumber
	deck := aNumber! !

!TurnLog methodsFor: 'event handling' stamp: 'MichaelOSullivan 7/22/2012 21:22'!
turnStarted: aNumber
	| stream |
	stream := String new writeStream.
	stream
		nextPutAll: log content;
		nextPutAll: 'Player ';
		print: aNumber;
		nextPutAll: '''s turn';
		crlf.
	deck = aNumber ifTrue: [ turn := turn + 1. ].
	log contents: stream contents.! !
